#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2015-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import enum

class TLSVersionRecordLayer(enum.IntEnum):
	"""TLS version as 16 bit integer (major is high byte, minor is low
	byte) as it is used on the record layer."""
	ProtocolTLSv1_0 = 0x0301
	ProtocolTLSv1_1 = 0x0302
	ProtocolTLSv1_2 = 0x0303

class TLSVersionHandshake(enum.IntEnum):
	"""TLS version as 16 bit integer (major is high byte, minor is low
	byte) as it is used inside the handshake protocol."""
	ProtocolSSLv3_0 = 0x0300
	ProtocolTLSv1_0 = 0x0301
	ProtocolTLSv1_1 = 0x0302
	ProtocolTLSv1_2 = 0x0303

class TLSVersion(enum.Enum):
	"""Combination of both to achieve a speicific version."""
	ProtocolTLSv1_0 = (TLSVersionRecordLayer.ProtocolTLSv1_0, TLSVersionHandshake.ProtocolTLSv1_0)
	ProtocolTLSv1_1 = (TLSVersionRecordLayer.ProtocolTLSv1_0, TLSVersionHandshake.ProtocolTLSv1_1)
	ProtocolTLSv1_2 = (TLSVersionRecordLayer.ProtocolTLSv1_0, TLSVersionHandshake.ProtocolTLSv1_2)
	ProtocolTLSv1_3 = (TLSVersionRecordLayer.ProtocolTLSv1_0, TLSVersionHandshake.ProtocolTLSv1_2)

class ContentType(enum.IntEnum):
	"""ContentType as Sect. 6.2.1. of RFC5246"""
	ChangeCipherSpec = 20
	Alert = 21
	Handshake = 22
	ApplicationData = 23
	Heartbeat = 24
	TLS12_CID = 25						#  Temporary, expires 2020-07-02

class HandshakeType(enum.IntEnum):
	"""HandshakeType as Sect. 7.4. of RFC5246"""
	HelloRequest = 0
	ClientHello = 1
	ServerHello = 2
	HelloVerifyRequest = 3
	NewSessionTicket = 4
	EndOfEarlyData = 5
	HelloRetryRequest = 6
	EncryptedExtensions = 8
	Certificate = 11
	ServerKeyExchange = 12
	CertificateRequest = 13
	ServerHelloDone = 14
	CertificateVerify = 15
	ClientKeyExchange = 16
	Finished = 20
	CertificateURL = 21
	CertificateStatus = 22
	SupplementalData = 23
	KeyUpdate = 24
	CompressedCertificate = 25
	MessageHash = 254

class ChangeCipherSpecType(enum.IntEnum):
	"""ChangeCipherSpec as Sect. 7.1. of RFC5246"""
	ChangeCipherSpec = 1

class TLSAlertLevel(enum.IntEnum):
	"""AlertLevel as Sect. 7.2. of RFC5246"""
	Warn = 1
	Fatal = 2

class TLSAlertDescription(enum.IntEnum):
	"""AlertDescription as Sect. 7.2. of RFC5246"""
	CloseNotify = 0
	UnexpectedMessage = 10
	BadRecordMAC = 20
	DecryptionFailedRESERVED = 21
	RecordOverflow = 22
	DecompressionFailure = 30
	HandshakeFailure = 40
	NoCertificateRESERVED = 41
	BadCertificate = 42
	UnsupportedCertificate = 43
	CertificateRevoked = 44
	CertificateExpired = 45
	CertificateUnknown = 46
	IllegalParameter = 47
	UnknownCA = 48
	AccessDenied = 49
	DecodeError = 50
	DecryptError = 51
	ExportRestrictionRESERVED = 60
	ProtocolVersion = 70
	InsufficientSecurity = 71
	InternalError = 80
	UserCanceled = 90
	NoRenegotiation = 100
	UnsupportedExtension = 110
	CertificateUnobtainable = 111
	UnrecognizedName = 112
	BadCertificateStatusResponse = 113
	BadCertificateHashValue = 114
	UnknownPSKIdentity = 115
	CertificateRequired = 116
	NoApplicationProtocol = 120

class CipherSuite(enum.IntEnum):
	"""Cipher Suites as defined in various RFCs."""
	# RFC5246 A.5
	TLS_NULL_WITH_NULL_NULL = 0x0
	TLS_RSA_WITH_NULL_MD5 = 0x01
	TLS_RSA_WITH_NULL_SHA = 0x02
	TLS_RSA_WITH_NULL_SHA256 = 0x3B
	TLS_RSA_WITH_RC4_128_MD5 = 0x04
	TLS_RSA_WITH_RC4_128_SHA = 0x05
	TLS_RSA_WITH_3DES_EDE_CBC_SHA = 0x0A
	TLS_RSA_WITH_AES_128_CBC_SHA = 0x2F
	TLS_RSA_WITH_AES_256_CBC_SHA = 0x35
	TLS_RSA_WITH_AES_128_CBC_SHA256 = 0x3C
	TLS_RSA_WITH_AES_256_CBC_SHA256 = 0x3D
	TLS_DH_DSS_WITH_3DES_EDE_CBC_SHA = 0x0D
	TLS_DH_RSA_WITH_3DES_EDE_CBC_SHA = 0x10
	TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA = 0x13
	TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA = 0x16
	TLS_DH_DSS_WITH_AES_128_CBC_SHA = 0x30
	TLS_DH_RSA_WITH_AES_128_CBC_SHA = 0x31
	TLS_DHE_DSS_WITH_AES_128_CBC_SHA = 0x32
	TLS_DHE_RSA_WITH_AES_128_CBC_SHA = 0x33
	TLS_DH_DSS_WITH_AES_256_CBC_SHA = 0x36
	TLS_DH_RSA_WITH_AES_256_CBC_SHA = 0x37
	TLS_DHE_DSS_WITH_AES_256_CBC_SHA = 0x38
	TLS_DHE_RSA_WITH_AES_256_CBC_SHA = 0x39
	TLS_DH_DSS_WITH_AES_128_CBC_SHA256 = 0x3E
	TLS_DH_RSA_WITH_AES_128_CBC_SHA256 = 0x3F
	TLS_DHE_DSS_WITH_AES_128_CBC_SHA256 = 0x40
	TLS_DHE_RSA_WITH_AES_128_CBC_SHA256 = 0x67
	TLS_DH_DSS_WITH_AES_256_CBC_SHA256 = 0x68
	TLS_DH_RSA_WITH_AES_256_CBC_SHA256 = 0x69
	TLS_DHE_DSS_WITH_AES_256_CBC_SHA256 = 0x6A
	TLS_DHE_RSA_WITH_AES_256_CBC_SHA256 = 0x6B
	TLS_DH_anon_WITH_RC4_128_MD5 = 0x18
	TLS_DH_anon_WITH_3DES_EDE_CBC_SHA = 0x1B
	TLS_DH_anon_WITH_AES_128_CBC_SHA = 0x34
	TLS_DH_anon_WITH_AES_256_CBC_SHA = 0x3A
	TLS_DH_anon_WITH_AES_128_CBC_SHA256 = 0x6C
	TLS_DH_anon_WITH_AES_256_CBC_SHA256 = 0x6D

	# RFC5932
	TLS_RSA_WITH_CAMELLIA_128_CBC_SHA = 0x41
	TLS_DH_DSS_WITH_CAMELLIA_128_CBC_SHA = 0x42
	TLS_DH_RSA_WITH_CAMELLIA_128_CBC_SHA = 0x43
	TLS_DHE_DSS_WITH_CAMELLIA_128_CBC_SHA = 0x44
	TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA = 0x45
	TLS_DH_anon_WITH_CAMELLIA_128_CBC_SHA = 0x46

	TLS_RSA_WITH_CAMELLIA_256_CBC_SHA = 0x84
	TLS_DH_DSS_WITH_CAMELLIA_256_CBC_SHA = 0x85
	TLS_DH_RSA_WITH_CAMELLIA_256_CBC_SHA = 0x86
	TLS_DHE_DSS_WITH_CAMELLIA_256_CBC_SHA = 0x87
	TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA = 0x88
	TLS_DH_anon_WITH_CAMELLIA_256_CBC_SHA = 0x89

	TLS_RSA_WITH_CAMELLIA_128_CBC_SHA256 = 0xBA
	TLS_DH_DSS_WITH_CAMELLIA_128_CBC_SHA256 = 0xBB
	TLS_DH_RSA_WITH_CAMELLIA_128_CBC_SHA256 = 0xBC
	TLS_DHE_DSS_WITH_CAMELLIA_128_CBC_SHA256 = 0xBD
	TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA256 = 0xBE
	TLS_DH_anon_WITH_CAMELLIA_128_CBC_SHA256 = 0xBF

	TLS_RSA_WITH_CAMELLIA_256_CBC_SHA256 = 0xC0
	TLS_DH_DSS_WITH_CAMELLIA_256_CBC_SHA256 = 0xC1
	TLS_DH_RSA_WITH_CAMELLIA_256_CBC_SHA256 = 0xC2
	TLS_DHE_DSS_WITH_CAMELLIA_256_CBC_SHA256 = 0xC3
	TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA256 = 0xC4
	TLS_DH_anon_WITH_CAMELLIA_256_CBC_SHA256 = 0xC5

	# RFC5288
	TLS_RSA_WITH_AES_128_GCM_SHA256 = 0x9C
	TLS_RSA_WITH_AES_256_GCM_SHA384 = 0x9D
	TLS_DHE_RSA_WITH_AES_128_GCM_SHA256 = 0x9E
	TLS_DHE_RSA_WITH_AES_256_GCM_SHA384 = 0x9F
	TLS_DH_RSA_WITH_AES_128_GCM_SHA256 = 0xA0
	TLS_DH_RSA_WITH_AES_256_GCM_SHA384 = 0xA1
	TLS_DHE_DSS_WITH_AES_128_GCM_SHA256 = 0xA2
	TLS_DHE_DSS_WITH_AES_256_GCM_SHA384 = 0xA3
	TLS_DH_DSS_WITH_AES_128_GCM_SHA256 = 0xA4
	TLS_DH_DSS_WITH_AES_256_GCM_SHA384 = 0xA5
	TLS_DH_anon_WITH_AES_128_GCM_SHA256 = 0xA6
	TLS_DH_anon_WITH_AES_256_GCM_SHA384 = 0xA7

	# RFC4785
	TLS_PSK_WITH_NULL_SHA = 0x2c
	TLS_DHE_PSK_WITH_NULL_SHA = 0x2d
	TLS_RSA_PSK_WITH_NULL_SHA = 0x2e

	# RFC5289
	TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256  = 0xC023
	TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384  = 0xC024
	TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA256 = 0xC025
	TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA384 = 0xC026
	TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256 = 0xC027
	TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384 = 0xC028
	TLS_ECDH_RSA_WITH_AES_128_CBC_SHA256 = 0xC029
	TLS_ECDH_RSA_WITH_AES_256_CBC_SHA384 = 0xC02A
	TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256  = 0xC02B
	TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384  = 0xC02C
	TLS_ECDH_ECDSA_WITH_AES_128_GCM_SHA256 = 0xC02D
	TLS_ECDH_ECDSA_WITH_AES_256_GCM_SHA384 = 0xC02E
	TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256 = 0xC02F
	TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384 = 0xC030
	TLS_ECDH_RSA_WITH_AES_128_GCM_SHA256 = 0xC031
	TLS_ECDH_RSA_WITH_AES_256_GCM_SHA384 = 0xC032

	# RFC4492
	TLS_ECDH_ECDSA_WITH_NULL_SHA = 0xC001
	TLS_ECDH_ECDSA_WITH_RC4_128_SHA = 0xC002
	TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA = 0xC003
	TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA = 0xC004
	TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA = 0xC005

	TLS_ECDHE_ECDSA_WITH_NULL_SHA = 0xC006
	TLS_ECDHE_ECDSA_WITH_RC4_128_SHA = 0xC007
	TLS_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA  = 0xC008
	TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA = 0xC009
	TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA = 0xC00A

	TLS_ECDH_RSA_WITH_NULL_SHA = 0xC00B
	TLS_ECDH_RSA_WITH_RC4_128_SHA = 0xC00C
	TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA = 0xC00D
	TLS_ECDH_RSA_WITH_AES_128_CBC_SHA = 0xC00E
	TLS_ECDH_RSA_WITH_AES_256_CBC_SHA = 0xC00F

	TLS_ECDHE_RSA_WITH_NULL_SHA = 0xC010
	TLS_ECDHE_RSA_WITH_RC4_128_SHA = 0xC011
	TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA = 0xC012
	TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA = 0xC013
	TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA = 0xC014

	TLS_ECDH_anon_WITH_NULL_SHA = 0xC015
	TLS_ECDH_anon_WITH_RC4_128_SHA = 0xC016
	TLS_ECDH_anon_WITH_3DES_EDE_CBC_SHA = 0xC017
	TLS_ECDH_anon_WITH_AES_128_CBC_SHA = 0xC018
	TLS_ECDH_anon_WITH_AES_256_CBC_SHA = 0xC019

	# RFC4162
	TLS_RSA_WITH_SEED_CBC_SHA = 0x96
	TLS_DH_DSS_WITH_SEED_CBC_SHA = 0x97
	TLS_DH_RSA_WITH_SEED_CBC_SHA = 0x98
	TLS_DHE_DSS_WITH_SEED_CBC_SHA = 0x99
	TLS_DHE_RSA_WITH_SEED_CBC_SHA = 0x9A
	TLS_DH_anon_WITH_SEED_CBC_SHA = 0x9B

	# RFC5469
	TLS_RSA_WITH_DES_CBC_SHA = 0x9
	TLS_DH_DSS_WITH_DES_CBC_SHA = 0xC
	TLS_DH_RSA_WITH_DES_CBC_SHA = 0xF
	TLS_DHE_DSS_WITH_DES_CBC_SHA = 0x12
	TLS_DHE_RSA_WITH_DES_CBC_SHA = 0x15
	TLS_DH_anon_WITH_DES_CBC_SHA = 0x1A
	TLS_RSA_WITH_IDEA_CBC_SHA = 0x7

	# RFC4346
	TLS_RSA_EXPORT_WITH_RC4_40_MD5 = 0x3
	TLS_RSA_EXPORT_WITH_RC2_CBC_40_MD5 = 0x6
	TLS_RSA_EXPORT_WITH_DES40_CBC_SHA = 0x8
	TLS_DH_DSS_EXPORT_WITH_DES40_CBC_SHA = 0xB
	TLS_DH_RSA_EXPORT_WITH_DES40_CBC_SHA = 0xE
	TLS_DHE_DSS_EXPORT_WITH_DES40_CBC_SHA = 0x11
	TLS_DHE_RSA_EXPORT_WITH_DES40_CBC_SHA = 0x14
	TLS_DH_anon_EXPORT_WITH_RC4_40_MD5 = 0x17
	TLS_DH_anon_EXPORT_WITH_DES40_CBC_SHA = 0x19
	TLS_KRB5_WITH_DES_CBC_SHA = 0x1E
	TLS_KRB5_WITH_3DES_EDE_CBC_SHA = 0x1F
	TLS_KRB5_WITH_RC4_128_SHA = 0x20
	TLS_KRB5_WITH_IDEA_CBC_SHA = 0x21
	TLS_KRB5_WITH_DES_CBC_MD5 = 0x22
	TLS_KRB5_WITH_3DES_EDE_CBC_MD5 = 0x23
	TLS_KRB5_WITH_RC4_128_MD5 = 0x24
	TLS_KRB5_WITH_IDEA_CBC_MD5 = 0x25
	TLS_KRB5_EXPORT_WITH_DES_CBC_40_SHA = 0x26
	TLS_KRB5_EXPORT_WITH_RC2_CBC_40_SHA = 0x27
	TLS_KRB5_EXPORT_WITH_RC4_40_SHA = 0x28
	TLS_KRB5_EXPORT_WITH_DES_CBC_40_MD5 = 0x29
	TLS_KRB5_EXPORT_WITH_RC2_CBC_40_MD5 = 0x2A
	TLS_KRB5_EXPORT_WITH_RC4_40_MD5 = 0x2B

	# draft-ietf-tls-openpgp-02
	TLS_PGP_DHE_DSS_WITH_CAST_CBC_SHA = 0x101
	TLS_PGP_DHE_DSS_WITH_IDEA_CBC_SHA = 0x102
	TLS_PGP_DHE_DSS_WITH_3DES_EDE_CBC_SHA = 0x103
	TLS_PGP_DHE_DSS_WITH_CAST_CBC_RMD = 0x104
	TLS_PGP_DHE_DSS_WITH_IDEA_CBC_RMD = 0x105
	TLS_PGP_DHE_DSS_WITH_3DES_EDE_CBC_RMD = 0x106
	TLS_PGP_DHE_RSA_WITH_CAST_CBC_SHA = 0x110
	TLS_PGP_RSA_WITH_CAST_CBC_SHA = 0x120
	TLS_PGP_RSA_WITH_IDEA_CBC_SHA = 0x121
	TLS_PGP_RSA_WITH_3DES_EDE_CBC_SHA = 0x122
	TLS_PGP_RSA_WITH_CAST_CBC_RMD = 0x123
	TLS_PGP_RSA_WITH_IDEA_CBC_RMD = 0x124
	TLS_PGP_RSA_WITH_3DES_EDE_CBC_RMD = 0x125
	TLS_PGP_DSA_WITH_NULL_SHA = 0x1F0

	# RFC7905
	TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256 = 0xCCA8
	TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256 = 0xCCA9
	TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256 = 0xCCAA
	TLS_PSK_WITH_CHACHA20_POLY1305_SHA256 = 0xCCAB
	TLS_ECDHE_PSK_WITH_CHACHA20_POLY1305_SHA256 = 0xCCAC
	TLS_DHE_PSK_WITH_CHACHA20_POLY1305_SHA256 = 0xCCAD
	TLS_RSA_PSK_WITH_CHACHA20_POLY1305_SHA256 = 0xCCAE

	# RFC5746
	TLS_EMPTY_RENEGOTIATION_INFO_SCSV = 0xff

	# RFC7507
	TLS_FALLBACK_SCSV = 0x5600

class CompressionMethod(enum.IntEnum):
	"""Compression method byte."""
	Null = 0
	Deflate = 1
	LZS = 64

class ExtensionType(enum.IntEnum):
	"""TLS extension types for hello packets."""
	server_name = 0									# [RFC6066]
	max_fragment_length = 1							# [RFC6066]
	client_certificate_url = 2						# [RFC6066]
	trusted_ca_keys = 3								# [RFC6066]
	truncated_hmac = 4								# [RFC6066]
	status_request = 5							 	# [RFC6066]
	user_mapping = 6								# [RFC4681]
	client_authz = 7								# [RFC5878]
	server_authz = 8								# [RFC5878]
	cert_type = 9									# [RFC6091]
	supported_groups = 10							# [RFC8422][RFC7919] (previously "elliptic_curves")
	ec_point_formats = 11							# [RFC4492]
	srp = 12										# [RFC5054]
	signature_algorithms = 13					 	# [RFC5246]
	use_srtp = 14									# [RFC5764]
	heartbeat = 15									# [RFC6520]
	application_layer_protocol_negotiation = 16		# [RFC7301]
	status_request_v2 = 17							# [RFC6961]
	signed_certificate_timestamp = 18				# [RFC6962]
	client_certificate_type = 19					# [RFC7250]
	server_certificate_type = 20					# [RFC7250]
	padding = 21									# [RFC7685]
	encrypt_then_mac = 22							# [RFC7366]
	extended_master_secret = 23						# [RFC7627]
	token_binding = 24								# [RFC8472]
	cached_info = 25								# [RFC7924]
	tls_lts = 26									# [draft-gutmann-tls-lts]
	compress_certificate = 27						# [RFC-ietf-tls-certificate-compression-09]
	record_size_limit = 28							# [RFC8449]
	pwd_protect = 29								# [RFC8492]
	pwd_clear = 30									# [RFC8492]
	password_salt = 31								# [RFC8492]
	ticket_pinning = 32								# [RFC8672]
	tls_cert_with_extern_psk = 34					# [RFC-ietf-tls-tls13-cert-with-extern-psk-07]
	session_ticket = 35								# [RFC5077][RFC8447] (previously "SessionTicket TLS")
	pre_shared_key = 41								# [RFC8446]
	early_data = 42									# [RFC8446]
	supported_versions = 43							# [RFC8446]
	cookie = 44										# [RFC8446]
	psk_key_exchange_modes = 45						# [RFC8446]
	certificate_authorities = 47					# [RFC8446]
	oid_filters = 48								# [RFC8446]
	post_handshake_auth = 49						# [RFC8446]
	signature_algorithms_cert = 50					# [RFC8446]
	key_share = 51									# [RFC8446]
	transparency_info = 52							# [draft-ietf-trans-rfc6962-bis]
	connection_id = 53								# Temporary, expires 2020-07-02
	external_id_hash = 55							# [RFC-ietf-mmusic-sdp-uks-07]
	external_session_id = 56						# [RFC-ietf-mmusic-sdp-uks-07]
	renegotiation_info = 65281						# [RFC5746]

class HashAlgorithm(enum.IntEnum):
	# RFC 5246 7.4.1.4.1
	none = 0
	md5 = 1
	sha1 = 2
	sha224 = 3
	sha256 = 4
	sha384 = 5
	sha512 = 6
	Intrinsic = 8

class SignatureAlgorithm(enum.IntEnum):
	# RFC 5246 7.4.1.4.1
	RSA = 1
	DSA = 2
	ECDSA = 3
	ED25519 = 7
	ED448 = 8
	GOSTr34102012_256 = 64
	GOSTr34102012_512 = 65

class KeyExchangeAlgorithm(enum.IntEnum):
	# RFC 5246 7.4.3
	DHE_DSS = 1
	DHE_RSA = 2
	DH_ANON = 3
	RSA = 4
	DH_DSS = 5
	DH_RSA = 6

class SupportedGroups(enum.IntEnum):
	# RFC 4492 5.1.1
	sect163k1 = 1
	sect163r1 = 2
	sect163r2 = 3
	sect193r1 = 4
	sect193r2 = 5
	sect233k1 = 6
	sect233r1 = 7
	sect239k1 = 8
	sect283k1 = 9
	sect283r1 = 10
	sect409k1 = 11
	sect409r1 = 12
	sect571k1 = 13
	sect571r1 = 14
	secp160k1 = 15
	secp160r1 = 16
	secp160r2 = 17
	secp192k1 = 18
	secp192r1 = 19
	secp224k1 = 20
	secp224r1 = 21
	secp256k1 = 22
	secp256r1 = 23
	secp384r1 = 24
	secp521r1 = 25
	arbitrary_explicit_prime_curves = 0xFF01
	arbitrary_explicit_char2_curves = 0xFF02
	X25519 = 0x1d
	X448 = 0x1e

class ECPointFormats(enum.IntEnum):
	# RFC 4492 5.1.2
	uncompressed = 0
	ansiX962_compressed_prime = 1
	ansiX962_compressed_char2 = 2

class ServerNameType(enum.IntEnum):
	"""Server name indication TLS extension server type."""
	Hostname = 0

class CertificateCompressionMethod(enum.IntEnum):
	# draft-ietf-tls-certificate-compression-09
	zlib = 1
	brotli = 2
	zstd = 3



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2020-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from x509sak.BaseAction import BaseAction
from x509sak.tls.Enums import TLSVersion
from x509sak.tls.MessageHelper import ClientHelloHelper
from x509sak.tls.TLSStructs import ClientHelloPkt, CertificatePkt
from x509sak.tls.TLSConnection import TLSClientConnection
from x509sak.X509Certificate import X509Certificate

class ActionTLSClient(BaseAction):
	def __init__(self, cmdname, args):
		BaseAction.__init__(self, cmdname, args)

		tls_version = TLSVersion.ProtocolTLSv1_2
		self._conn = TLSClientConnection.tcp_connect(tls_version = tls_version, servername = args.servername, port = args.port)
		if self._args.starttls is None:
			pass
		elif self._args.starttls == "smtp":
			self._starttls_smtp(self._conn.transport)
		else:
			raise NotImplementedError(self._args.starttls)
		self._conn.decoder.add_hook("handshake", self._recv_handshake)
		self._conn.decoder.add_hook("record_layer", self._recv_record_layer)

		chh = ClientHelloHelper()
		client_hello = chh.create(server_name = args.servername)
		frame = ClientHelloPkt.pack(client_hello)
		self._conn.send_handshake(frame)
		self._conn.receive()

	def _starttls_smtp(self, conn):
		conn.recvline()
		conn.send(b"HELO foobar\r\n")
		conn.recvline()
		conn.send(b"STARTTLS\r\n")
		conn.recvline()

	def _recv_record_layer(self, hooktype, msg_id, data):
		print("<- %3d %s %s (%d bytes)" % (msg_id, data["content_type"].name, data["record_layer_version"], len(data["payload"])))

	def _recv_handshake(self, hooktype, msg_id, msg_type, data):
		print("<= %3d %s" % (msg_id, msg_type.name))
		if msg_type == CertificatePkt:
			print("    %d certificates received:" % (len(data["payload"]["certificates"])))
			for der_data in data["payload"]["certificates"]:
				cert = X509Certificate(bytes(der_data))
				print("        %s" % (cert))



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2018 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from x509sak.BaseAction import BaseAction
from x509sak.RSAPrivateKey import RSAPrivateKey
from x509sak.ECPrivateKey import ECPrivateKey
from x509sak.EDPrivateKey import EDPrivateKey
from x509sak.PublicKey import PublicKey
from x509sak.AlgorithmDB import Cryptosystems

class ActionDumpKey(BaseAction):
	def __init__(self, cmdname, args):
		BaseAction.__init__(self, cmdname, args)
		if self._args.public_key:
			key = PublicKey.read_pemfile(self._args.key_filename)[0]
			if key.pk_alg.value.cryptosystem == Cryptosystems.RSA:
				print ("# %d bit RSA public key (ID %s)" % (key.n.bit_length(), key.keyid().hex()))
				print("n = 0x%x" % (key.n))
				print("e = 0x%x" % (key.e))
			elif key.pk_alg.value.cryptosystem == Cryptosystems.ECC_ECDSA:
				print("# ECC public key on %s (key ID %s)" % (key.curve.name, key.keyid().hex()))
				print("curve_name = \"%s\"" % (key.curve.name))
				print("(x, y) = (0x%x, 0x%x)" % (key.x, key.y))
			elif key.pk_alg.value.cryptosystem == Cryptosystems.ECC_EdDSA:
				print("# ECC public key on Twisted Edwards curve %s %s prehashing (key ID %s)" % (key.curve.name, "with" if key.prehash else "without", key.keyid().hex()))
				print("curve_name = \"%s\"" % (key.curve.name))
				print("prehash = %s" % (key.prehash))
				print("(x, y) = (0x%x, 0x%x)" % (key.x, key.y))
			else:
				raise NotImplementedError(key.pk_alg.value.cryptosystem)
		else:
			if self._args.key_type == "rsa":
				key = RSAPrivateKey.read_pemfile(self._args.key_filename)[0]
				print ("# %d bit RSA private key (ID %s)" % (key.n.bit_length(), key.pubkey.keyid().hex()))
				print("p = 0x%x" % (key.p))
				print("q = 0x%x" % (key.q))
				print("n = p * q")
				print("e = 0x%x" % (key.e))
				print("d = 0x%x" % (key.d))
			elif self._args.key_type == "ecc":
				key = ECPrivateKey.read_pemfile(self._args.key_filename)[0]
				print("# ECC private key on %s (key ID %s)" % (key.curve.name, key.pubkey.keyid().hex()))
				print("curve_name = \"%s\"" % (key.curve.name))
				print("d = 0x%x" % (key.d))
				print("(x, y) = (0x%x, 0x%x)" % (key.x, key.y))
			elif self._args.key_type == "eddsa":
				key = EDPrivateKey.read_pemfile(self._args.key_filename)[0]
				pubkey = key.pubkey
				print("# ECC private key on Twisted Edwards curve %s %s prehashing (key ID %s)" % (key.curve.name, "with" if key.prehash else "without", pubkey.keyid().hex()))
				print("curve_name = \"%s\"" % (key.curve.name))
				print("prehash = %s" % (key.prehash))
				print("priv = bytes.fromhex(\"%s\")" % (key.priv.hex()))
				print("hashfnc = hashlib.new(\"%s\")" % (key.curve.expand_hashfnc))
				print("(expand_bitwise_and, expand_bitwise_or) = (0x%x, 0x%x)" % (key.curve.expand_bitwise_and, key.curve.expand_bitwise_or))
				print("a = 0x%x" % (key.scalar))
				print("(x, y) = (0x%x, 0x%x)" % (pubkey.x, pubkey.y))
			else:
				raise NotImplementedError(self._args.key_type)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from x509sak.Exceptions import InvalidInternalDataException

class BijectiveDictException(InvalidInternalDataException): pass

class BijectiveDict():
	def __init__(self, values, key_predicate = None, value_predicate = None):
		self._key_predicate = key_predicate if (key_predicate is not None) else lambda key: key
		self._value_predicate = value_predicate if (value_predicate is not None) else lambda value: value
		self._lookup_dict = { self._key_predicate(key): value for (key, value) in values.items() }
		self._lookup_revdict = { self._value_predicate(value): key for (key, value) in values.items() }
		if len(values) != len(self._lookup_dict):
			raise BijectiveDictException("Dictionary has duplicate keys after application of key predicate.")

		if len(values) != len(self._lookup_revdict):
			raise BijectiveDictException("Dictionary not bijective.")

	def keys(self):
		return self._lookup_dict.keys()

	def inverse(self, value):
		value = self._value_predicate(value)
		return self._lookup_revdict[value]

	def get(self, key, surrogate = None):
		key = self._key_predicate(key)
		return self._lookup_dict.get(key, surrogate)

	def __getitem__(self, key):
		key = self._key_predicate(key)
		return self._lookup_dict[key]

	def __len__(self):
		return len(self._lookup_dict)

	def __iter__(self):
		return iter(self._lookup_dict)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import pyasn1.type.char
from x509sak.OID import OIDDB
from x509sak.estimate import Compatibility
from x509sak.estimate.Judgement import Commonness, RFCReference, LiteratureReference
from x509sak.estimate.Validator import BaseValidationResult, BaseValidator

class DistinguishedNameValidationResult(BaseValidationResult):
	_VALID_ALPHABETS = {
		pyasn1.type.char.PrintableString:		set("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 '()+,-./:=?"),
	}
	_LARGE_RDN_AMOUNT = 20		# Warn if more than this amount of RDNs are present
	_MAX_LENGTH = {
		OIDDB.RDNTypes.inverse("CN"):			64,
		OIDDB.RDNTypes.inverse("C"):			2,
		OIDDB.RDNTypes.inverse("emailAddress"):	255,
		OIDDB.RDNTypes.inverse("GN"):			16,
		OIDDB.RDNTypes.inverse("initials"):		5,
		OIDDB.RDNTypes.inverse("UID"):			32,
		OIDDB.RDNTypes.inverse("OU"):			32,
		OIDDB.RDNTypes.inverse("O"):			64,
		OIDDB.RDNTypes.inverse("postalCode"):	16,
		OIDDB.RDNTypes.inverse("SN"):			40,
		OIDDB.RDNTypes.inverse("x121Address"):	16,
	}

	def _validate_rdn_component(self, rdn_item):
		asn1type = type(rdn_item.asn1)
		if asn1type in self._VALID_ALPHABETS:
			valid_chars = self._VALID_ALPHABETS[asn1type]
			illegal_chars = set(rdn_item.printable_value) - valid_chars
			if len(illegal_chars) > 0:
				self._report("Enc_DER_Struct_DN_RDN_IllegalCharacter", "distinguished name contains character(s) \"%s\" which are invalid for a %s at element \"%s\"." % ("".join(sorted(illegal_chars)), asn1type.__name__, OIDDB.RDNTypes.get(rdn_item.oid, str(rdn_item.oid))), compatibility = Compatibility.STANDARDS_DEVIATION)

		if isinstance(rdn_item.asn1, pyasn1.type.char.TeletexString):
			self._report("Enc_DER_Struct_DN_RDN_DeprecatedType", "distinguished name contains deprecated TeletexString at element \"%s\"." % (OIDDB.RDNTypes.get(rdn_item.oid, str(rdn_item.oid))), compatibility = Compatibility.STANDARDS_DEVIATION)

		if not rdn_item.decodable:
			self._report("Enc_DER_Struct_DN_RDN_Malformed", "distinguished name contains undecodable RDN element \"%s\"." % (OIDDB.RDNTypes.get(rdn_item.oid, str(rdn_item.oid))), commonness = Commonness.HIGHLY_UNUSUAL)
		else:
			max_length = self._MAX_LENGTH.get(rdn_item.oid)
			if max_length is not None:
				if len(rdn_item.printable_value) > max_length:
					standard = RFCReference(rfcno = 5280, sect = "A.1", verb = "MUST", text = "specifications of Upper Bounds MUST be regarded as mandatory from Annex B of ITU-T X.411 Reference Definition of MTS Parameter Upper Bounds")
					self._report("Enc_DER_Struct_DN_RDN_LengthExceeded", "distinguished name contains RDN element \"%s\" which is supposed to have a maximum length of %d characters, but actually has a length of %d characters." % (OIDDB.RDNTypes.get(rdn_item.oid, str(rdn_item.oid)), max_length, len(rdn_item.printable_value)), commonness = Commonness.UNUSUAL, compatibility = Compatibility.STANDARDS_DEVIATION, standard = standard)


		if not rdn_item.printable:
			# TODO standards reference?
			self._report("Enc_DER_Struct_DN_RDN_NonPrintable", "distinguished name contains RDN element item \"%s\" (ASN.1 type %s) which is not printable." % (OIDDB.RDNTypes.get(rdn_item.oid, str(rdn_item.oid)), rdn_item.asn1.__class__.__name__), commonness = Commonness.HIGHLY_UNUSUAL, compatibility = Compatibility.STANDARDS_DEVIATION)

		else:
			if rdn_item.printable_value == "":
				self._report("Enc_DER_Struct_DN_RDN_EmptyString", "contains relative distinguished name with empty string value.", commonness = Commonness.HIGHLY_UNUSUAL)

	def _validate_rdn(self, rdn):
		if rdn.component_cnt > 1:
			self._report("Enc_DER_Struct_DN_RDN_MultiValuedRDN", "distinguished name contains a multivalue RDN: %s" % (rdn.pretty_str), commonness = Commonness.HIGHLY_UNUSUAL, compatibility = Compatibility.LIMITED_SUPPORT)

		rdn_data = [ (rdn_item.oid, rdn_item.derdata) for rdn_item in rdn ]
		rdn_data_set = set(rdn_data)
		if len(rdn_data) != len(rdn_data_set):
			self._report("Enc_DER_Struct_DN_RDN_DuplicateSet_Key_Value", "relative distinguished name contains identical key/value more than once in SET: %s" % (rdn.pretty_str), commonness = Commonness.HIGHLY_UNUSUAL, compatibility = Compatibility.LIMITED_SUPPORT)

		rdn_oids = [ rdn_item.oid for rdn_item in rdn ]
		rdn_oids_set = set(rdn_oids)
		if len(rdn_oids) != len(rdn_oids_set):
			standard = LiteratureReference(author = "ITU-T", title = "Recommendation X.501: Information technology - Open Systems Interconnection – The Directory: Models", sect = "9.3", month = 8, year = 2005, quote = "The set that forms an RDN contains exactly one AttributeTypeAndDistinguishedValue for each attribute which contains distinguished values in the entry; that is, a given attribute type cannot appear twice in the same RDN.")
			self._report("Enc_DER_Struct_DN_RDN_DuplicateSet_Key", "multivalued relative distinguished name contains same key OID more than once in: %s" % (rdn.pretty_str), commonness = Commonness.HIGHLY_UNUSUAL, compatibility = Compatibility.STANDARDS_DEVIATION, standard = standard)

		for rdn_item in rdn:
			self._validate_rdn_component(rdn_item)

	def _validate(self):
		if self._subject.rdn_count == 0:
			self._report("Enc_DER_Struct_DN_Empty", "contains no relative distinguished names (RDNs).", commonness = Commonness.HIGHLY_UNUSUAL)

		all_cns = self._subject.get_all(OIDDB.RDNTypes.inverse("CN"))
		if len(all_cns) == 0:
			self._report("Enc_DER_Struct_DN_NoCN", "does not have any common name (CN) set.", commonness = Commonness.HIGHLY_UNUSUAL)
		elif len(all_cns) > 1:
			self._report("Enc_DER_Struct_DN_MultipleCN", "does have more than one common name (CN) set; in particular, %d CN fields were encountered." % (len(all_cns)), commonness = Commonness.UNUSUAL)

		seen_oid_keys = set()
		for rdn in self._subject:
			self._validate_rdn(rdn)

			oidkey = rdn.oidkey
			if oidkey in seen_oid_keys:
				oidkey_str = " + ".join(OIDDB.RDNTypes.get(oid, str(oid)) for oid in oidkey)
				self._report("Enc_DER_Struct_DN_DuplicateRDNs", "contains RDN element at least twice: %s (at element %s)" % (oidkey_str, rdn.pretty_str), commonness = Commonness.UNUSUAL, compatibility = Compatibility.LIMITED_SUPPORT)
			else:
				seen_oid_keys.add(oidkey)

		if self._subject.rdn_count > self._LARGE_RDN_AMOUNT:
			self._report("Enc_DER_Struct_DN_UnusuallyManyRDNs", "contains an unusually high amount of RDNs (%d)." % (self._subject.rdn_count), commonness = Commonness.UNUSUAL)

class DistinguishedNameValidator(BaseValidator):
	_ValidationResultClass = DistinguishedNameValidationResult



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2019-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from x509sak.HexDump import HexDump

class DataBufferException(Exception): pass
class NotEnoughDataException(DataBufferException): pass

class _RewindObject():
	def __init__(self, data_buffer):
		self._data_buffer = data_buffer
		self._pos = None

	def __enter__(self):
		assert(self._pos is None)
		self._pos = self._data_buffer.offset

	def __exit__(self, exception_class, exception, traceback):
		if exception_class is not None:
			# Exception was thrown, rewind.
			self._data_buffer.offset = self._pos

class DataBuffer():
	def __init__(self, initial_value = None):
		if initial_value is None:
			self._data = bytearray()
		else:
			self._data = bytearray(initial_value)
		self._offset = 0

	@classmethod
	def fromhex(cls, hex_str):
		return cls(bytes.fromhex(hex_str))

	@property
	def offset(self):
		return self._offset

	@offset.setter
	def offset(self, value):
		assert(0 <= value <= self.length)
		self._offset = value

	@property
	def length(self):
		return len(self._data)

	@property
	def remaining(self):
		return self.length - self.offset

	def rewind_on_exception(self):
		return _RewindObject(self)

	def get(self, length):
		if length > self.remaining:
			raise NotEnoughDataException("%d bytes requested from data buffer, but only %d bytes remaining." % (length, self.remaining))
		returned_data = self._data[self.offset : self.offset + length]
		self._offset += length
		return returned_data

	def append(self, data):
		self += data

	def hexdump(self):
		print("%s" % (self))
		HexDump().dump(self._data)

	def __iadd__(self, other):
		self._data += other
		return self

	def __iter__(self):
		return iter(self._data)

	def __str__(self):
		return "DataBuffer<%d (0x%x) bytes, offset %d (0x%x), remaining %d (0x%x)>" % (self.length, self.length, self.offset, self.offset, self.remaining, self.remaining)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import collections
import datetime
import logging
from .X509Certificate import X509Certificate

_log = logging.getLogger("x509sak.CertificatePool")

class CertificatePool():
	_CertificateChain = collections.namedtuple("CertificateChain", [ "root", "chain", "leaf", "full_chain" ])
	def __init__(self):
		self._pool = collections.defaultdict(set)

	@property
	def certificate_count(self):
		return sum(len(certs) for certs in self._pool.values())

	def find_chain(self, certificate, max_depth = 100):
		now = datetime.datetime.utcnow()
		leaf = certificate
		chain = [ ]
		for _ in range(max_depth):
			issuers = list(self.find_issuers(certificate))
			if len(issuers) == 0:
				issuer = None
				break

			# If there's multiple issuers, prefer those which are currently
			# valid and, as a second criterion, those which are valid for the
			# longest time period.
			issuers.sort(key = lambda crt: (not crt.is_time_valid(now), -crt.seconds_until_expires(now)))
			issuer = issuers[0]
			if issuer == certificate:
				break
			chain.append(issuer)
			certificate = issuer

		full_chain = [ leaf ] + chain
		if issuer == certificate:
			# Got a root cert
			root = certificate
			chain = tuple(chain[:-1])
		else:
			# Partial chain match
			root = None
			chain = tuple(chain)
		return self._CertificateChain(root = root, chain = chain, leaf = leaf, full_chain = tuple(full_chain))

	def find_issuers(self, subject_cert):
		if subject_cert.issuer == subject_cert.subject:
			if subject_cert.signed_by(subject_cert):
				yield subject_cert

		if subject_cert.issuer in self._pool:
			for possible_issuer_cert in self._pool[subject_cert.issuer]:
				if possible_issuer_cert != subject_cert:
					if subject_cert.signed_by(possible_issuer_cert):
						yield possible_issuer_cert

	def add_certificate(self, cert):
		self._pool[cert.subject].add(cert)

	def add_certificates(self, certs):
		for cert in certs:
			self.add_certificate(cert)

	def load_pemfile(self, filename):
		certs = X509Certificate.read_pemfile(filename, ignore_errors = True)
		for cert in certs:
			self.add_certificate(cert)

	def load_pemdirectory(self, dirname, extensions = None):
		if extensions is None:
			extensions = set([ ".crt", ".pem" ])
		if not dirname.endswith("/"):
			dirname += "/"
		for filename in os.listdir(dirname):
			for extension in extensions:
				if filename.endswith(extension):
					break
			else:
				continue
			fullfilename = dirname + filename
			self.load_pemfile(fullfilename)

	def load_source(self, pathname, extensions = None):
		if os.path.isfile(pathname):
			_log.debug("Loading CA certificate from %s", pathname)
			self.load_pemfile(pathname)
		elif os.path.isdir(pathname):
			_log.debug("Loading CA certificates from directory %s", pathname)
			self.load_pemdirectory(pathname, extensions = extensions)
		else:
			raise Exception("Unable to load trust store source %s: No such file or directory" % (pathname))

	def load_sources(self, sources, extensions = None):
		for source in sources:
			self.load_source(source, extensions = extensions)

	def dump(self):
		for (subject, certs) in self._pool.items():
			print("Subject %s:" % (subject))
			for cert in certs:
				print("   %s" % (cert))

	def __iter__(self):
		for certs in list(self._pool.values()):
			for cert in list(certs):
				yield cert



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2018 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import random
from x509sak.BaseAction import BaseAction
from x509sak.PrimeDB import PrimeDB
from x509sak.NumberTheory import NumberTheory
from x509sak.DSAParameters import DSAParameters
from x509sak.Exceptions import UnfulfilledPrerequisitesException

class ActionGenerateBrokenDSA(BaseAction):
	def __init__(self, cmdname, args):
		BaseAction.__init__(self, cmdname, args)

		if (not self._args.force) and os.path.exists(self._args.outfile):
			raise UnfulfilledPrerequisitesException("File/directory %s already exists. Remove it first or use --force." % (self._args.outfile))

		self._prime_db = PrimeDB(self._args.prime_db, generator_program = self._args.generator)
		q = self._prime_db.get(args.N_bits)
		if self._args.verbose >= 1:
			print("Chosen q = 0x%x" % (q))

		bit_diff = args.L_bits - q.bit_length()
		while True:
			r = NumberTheory.randint_bits(bit_diff, two_msb_set = True)
			p = (r * q) + 1
			if NumberTheory.is_probable_prime(p):
				break
		if self._args.verbose >= 1:
			print("Chosen p = 0x%x" % (p))

		assert(q.bit_length() == args.N_bits)
		assert(p.bit_length() == args.L_bits)
		assert((p - 1) % q == 0)

		# Non-verifiable method of generating g, see A.2.1 of FIPS 186-4, pg. 41
		e = (p - 1) // q
		while True:
			h = random.randint(2, p - 2)
			g = pow(h, e, p)
			if g == 1:
				continue
			break

		if self._args.verbose >= 1:
			print("Chosen g = 0x%x" % (g))

		dsa_parameters = DSAParameters.create(p = p, q = q, g = g)
		dsa_parameters.write_pemfile(self._args.outfile)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import time
import collections

class ScrapeEngine():
	_Pattern = collections.namedtuple("Pattern", [ "callback", "pattern", "min_length", "max_length" ])

	def __init__(self, filename):
		self._filename = filename
		self._patterns = [ ]
		self._chunk_size = 1024 * 1024
		self._last_chunk_offsets = set()

	def search(self, callback, pattern, min_length = 0, max_length = 4096):
		pattern = self._Pattern(callback = callback, pattern = pattern, min_length = min_length, max_length = max_length)
		self._patterns.append(pattern)

	def _search_pattern(self, pattern, offset, data, ignore_tail_bytes):
		results = { }
		search_offset = 0
		while True:
			found_index = data.find(pattern.pattern, search_offset)
			if (found_index == -1) or (found_index >= len(data) - ignore_tail_bytes):
				break
			abs_offset = offset + found_index
			results[abs_offset] = pattern
			search_offset = found_index + 1
		return results

	def _search_chunk(self, start_offset, data, ignore_tail_bytes):
		results = { }
		for pattern in self._patterns:
			results.update(self._search_pattern(pattern, start_offset, data, ignore_tail_bytes))
		for (offset, pattern) in sorted(results.items()):
			if offset in self._last_chunk_offsets:
				continue
			relative_offset = offset - start_offset
			found_data = data[relative_offset : relative_offset + pattern.max_length]
			if len(found_data) > pattern.min_length:
				pattern.callback(offset, found_data)
		self._last_chunk_offsets = set(results.keys())

	def commence(self, start_offset = 0, length = None, progress_callback = None):
		t0 = time.time()
		trackback = max(pattern.max_length for pattern in self._patterns)

		chunk_count = 0
		with open(self._filename, "rb") as f:
			# EOF position = actual end-of-file
			f.seek(0, os.SEEK_END)
			eof_position = f.tell()

			if length is not None:
				# But can be shorter
				eof_position = min(eof_position, start_offset + length)
			f.seek(start_offset)

			# Then seek through whole file
			at_eof = False
			while not at_eof:
				offset = f.tell()
				chunk = f.read(self._chunk_size + trackback)
				at_eof = (offset + len(chunk)) >= eof_position
				if not at_eof:
					ignore_tail_bytes = trackback
				else:
					ignore_tail_bytes = 0
				self._search_chunk(offset, chunk, ignore_tail_bytes)
				chunk_count += 1
				if (progress_callback is not None) and ((chunk_count % 100) == 0):
					progress_callback(position = offset + len(chunk) - start_offset, total_length = eof_position - start_offset, elapsed_secs = time.time() - t0)
			end_offset = f.tell()
			progress_callback(position = end_offset - start_offset, total_length = eof_position - start_offset, elapsed_secs = time.time() - t0)
		return end_offset



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2018 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from x509sak.tests import BaseTest
from x509sak.SubprocessExecutor import SubprocessExecutor

class SubprocessExecutorTests(BaseTest):
	def test_basic_stdout(self):
		result = SubprocessExecutor([ "echo", "foobar" ]).run()
		self.assertEqual(result.stdout, b"foobar\n")
		self.assertEqual(result.stderr, b"")
		self.assertEqual(result.stdout_text, "foobar\n")
		self.assertEqual(result.stderr_text, "")
		self.assertEqual(result.stdouterr, b"foobar\n")
		self.assertEqual(result.stdouterr_text, "foobar\n")

	def test_basic_stdin(self):
		result = SubprocessExecutor([ "cat" ], stdin = b"mookoo").run()
		self.assertEqual(result.stdout, b"mookoo")
		self.assertEqual(result.stderr, b"")
		self.assertEqual(result.stdout_text, "mookoo")
		self.assertEqual(result.stderr_text, "")
		self.assertEqual(result.stdouterr, b"mookoo")
		self.assertEqual(result.stdouterr_text, "mookoo")

	def test_basic_stderr(self):
		result = SubprocessExecutor([ "cat", "/fdisohjfdiso" ], success_return_codes = [ 1 ]).run()
		self.assertEqual(result.stdout, b"")
		self.assertIn(b"such file or directory", result.stderr)
		self.assertEqual(result.stdout_text, "")
		self.assertIn("such file or directory", result.stderr_text)
		self.assertIn(b"such file or directory", result.stdouterr)
		self.assertIn("such file or directory", result.stdouterr_text)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2018 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import tempfile
import shutil
from x509sak import CAManager
from x509sak.BaseAction import BaseAction
from x509sak.PrivateKeyStorage import PrivateKeyStorage, PrivateKeyStorageForm
from x509sak.KeySpecification import KeySpecification
from x509sak.OpenSSLTools import OpenSSLTools
from x509sak.Exceptions import UnfulfilledPrerequisitesException, InvalidUsageException
from x509sak.EDPrivateKey import EDPrivateKey

class ActionCreateCA(BaseAction):
	def __init__(self, cmdname, args):
		BaseAction.__init__(self, cmdname, args)
		if self._args.force:
			try:
				os.unlink(self._args.capath)
			except (IsADirectoryError, FileNotFoundError):
				pass
			try:
				shutil.rmtree(self._args.capath)
			except FileNotFoundError:
				pass
		else:
			if os.path.exists(self._args.capath):
				raise UnfulfilledPrerequisitesException("File/directory %s already exists. Remove it first or use --force." % (self._args.capath))

		os.makedirs(self._args.capath)
		os.chmod(self._args.capath, 0o700)

		camgr = CAManager(self._args.capath)
		custom_x509_extensions = { custom_x509_extension.key: custom_x509_extension.value for custom_x509_extension in self._args.extension }
		if (self._args.gen_keyspec is not None) or ((self._args.gen_keyspec is None) and (self._args.hardware_key is None)):
			# We need to generate a key.
			if self._args.gen_keyspec is not None:
				keyspec = self._args.gen_keyspec
			else:
				# Cannot default to this in argparse because it's part of a
				# mutually exclusive group with hardware_key
				keyspec = KeySpecification.from_cmdline_str("ecc:secp384r1")
			private_key_storage = PrivateKeyStorage(PrivateKeyStorageForm.PEM_FILE, filename = "CA.key", search_path = camgr.capath)
			OpenSSLTools.create_private_key(private_key_storage, keyspec = keyspec)
		else:
			# The key is stored in hardware.
			private_key_storage = PrivateKeyStorage(PrivateKeyStorageForm.HARDWARE_TOKEN, pkcs11uri = self._args.hardware_key, so_search_path = self._args.pkcs11_so_search, module_so = self._args.pkcs11_module)

		signing_hash = self._args.hashfnc
		if private_key_storage.is_file_based:
			privkey = private_key_storage.load_private_key()
			if isinstance(privkey, EDPrivateKey):
				# We cannot specify a hash function for EdDSA because this
				# causes OpenSSL to fail with "elliptic curve
				# routines:pkey_ecd_ctrl:invalid digest type" -- for EdDSA, the
				# hash algorithms are implied
				signing_hash = None

		camgr.create_ca_structure(private_key_storage = private_key_storage, unique_subject = not self._args.allow_duplicate_subjects)
		if self._args.parent_ca is None:
			# Self-signed root CA
			camgr.create_selfsigned_ca_cert(subject_dn = self._args.subject_dn, validity_days = self._args.validity_days, custom_x509_extensions = custom_x509_extensions, signing_hash = signing_hash, serial = self._args.serial)

			# Create certificate chain file that only consists of our
			# self-signed certificate
			shutil.copy(self._args.capath + "/CA.crt", self._args.capath + "/chain.crt")
		else:
			# Intermediate CA
			if self._args.serial is not None:
				raise InvalidUsageException("Can only specify certificate serial number when creating self-signed root CA certificate.")
			with tempfile.NamedTemporaryFile("w", prefix = "ca_", suffix = ".csr") as csr:
				camgr.create_ca_csr(csr_filename = csr.name, subject_dn = self._args.subject_dn)
				parent_ca = CAManager(self._args.parent_ca)
				parent_ca.sign_csr(csr.name, camgr.root_crt_filename, subject_dn = self._args.subject_dn, validity_days = self._args.validity_days, custom_x509_extensions = custom_x509_extensions, extension_template = "ca", signing_hash = signing_hash)

			# Create a certificate chain by appending the parent chain to our certificate
			if os.path.isfile(self._args.parent_ca + "/chain.crt"):
				with open(self._args.parent_ca + "/chain.crt") as parent_chainfile:
					parent_chain = parent_chainfile.read()
				with open(self._args.capath + "/CA.crt") as new_certificate_file:
					new_certificate = new_certificate_file.read()
				with open(self._args.capath + "/chain.crt", "w") as intermediate_chainfile:
					intermediate_chainfile.write(parent_chain)
					intermediate_chainfile.write(new_certificate)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import sys

class FileWriter():
	def __init__(self, filename, mode = "w"):
		assert(isinstance(filename, str))
		assert(mode in [ "w", "wb" ])
		self._filename = filename
		self._mode = mode
		self._f = None

	def __enter__(self):
		assert(self._f is None)
		if self._filename != "-":
			self._f = open(self._filename, self._mode)
		else:
			if self._mode == "w":
				self._f = sys.stdout
			else:
				self._f = sys.stdout.buffer
		return self._f

	def __exit__(self, *args):
		if self._filename != "-":
			self._f.close()
		else:
			self._f.flush()



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2019-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import pyasn1.codec.der.decoder
import x509sak.ASN1Models as ASN1Models
from x509sak.OID import OID, OIDDB
from x509sak.AlgorithmDB import HashFunctions

class RSAPSSParameters():
	_TRAILER_FIELD_VALUES = {
		1:	0xbc,
	}

	def __init__(self, hash_algorithm_oid, mask_algorithm_oid, mask_hash_algorithm_oid, salt_length, trailer_field):
		self._hash_algorithm_oid = hash_algorithm_oid
		self._mask_algorithm_oid = mask_algorithm_oid
		self._mask_hash_algorithm_oid = mask_hash_algorithm_oid
		self._salt_length = salt_length
		self._trailer_field = trailer_field
		self._hash_algorithm = HashFunctions.lookup("oid", self.hash_algorithm_oid)
		self._mask_algorithm = OIDDB.RSAPSSMaskGenerationAlgorithm.get(self.mask_algorithm_oid)
		self._mask_hash_algorithm = HashFunctions.lookup("oid", self.mask_hash_algorithm_oid)

	@property
	def hash_algorithm_oid(self):
		return self._hash_algorithm_oid

	@property
	def mask_algorithm_oid(self):
		return self._mask_algorithm_oid

	@property
	def mask_hash_algorithm_oid(self):
		return self._mask_hash_algorithm_oid

	@property
	def hash_algorithm(self):
		return self._hash_algorithm

	@property
	def mask_algorithm(self):
		return self._mask_algorithm

	@property
	def mask_hash_algorithm(self):
		return self._mask_hash_algorithm

	@property
	def salt_length(self):
		return self._salt_length

	@property
	def trailer_field(self):
		return self._trailer_field

	@property
	def trailer_field_value(self):
		return self._TRAILER_FIELD_VALUES.get(self.trailer_field)

	@classmethod
	def from_asn1(cls, asn1):
		constructor_arguments = { }
		if asn1["hashAlgorithm"].hasValue():
			constructor_arguments["hash_algorithm_oid"] = OID.from_asn1(asn1["hashAlgorithm"]["algorithm"])
		else:
			# Default for RSASSA-PSS is SHA-1
			constructor_arguments["hash_algorithm_oid"] = OIDDB.HashFunctions.inverse("sha1")

		if asn1["maskGenAlgorithm"].hasValue():
			constructor_arguments["mask_algorithm_oid"] = OID.from_asn1(asn1["maskGenAlgorithm"]["algorithm"])
			(decoded_mask_parameters, tail) = pyasn1.codec.der.decoder.decode(asn1["maskGenAlgorithm"]["parameters"], asn1Spec = ASN1Models.HashAlgorithm())
			constructor_arguments["mask_hash_algorithm_oid"] = OID.from_asn1(decoded_mask_parameters["algorithm"])
		else:
			# Default for RSASSA-PSS is mgf1 with SHA-1
			constructor_arguments["mask_algorithm_oid"] = OIDDB.RSAPSSMaskGenerationAlgorithm.inverse("mgf1")
			constructor_arguments["mask_hash_algorithm_oid"] = OIDDB.HashFunctions.inverse("sha1")

		if asn1["saltLength"].hasValue():
			constructor_arguments["salt_length"] = int(asn1["saltLength"])
		else:
			constructor_arguments["salt_length"] = 20

		if asn1["trailerField"].hasValue():
			constructor_arguments["trailer_field"] = int(asn1["trailerField"])
		else:
			constructor_arguments["trailer_field"] = 1

		return cls(**constructor_arguments)

	def __str__(self):
		parts = [ ]
		parts.append("Hash=%s" % (self.hash_algorithm.name if (self.hash_algorithm is not None) else self.hash_algorithm_oid))
		parts.append("Mask=%s" % (self.mask_algorithm if (self.mask_algorithm is not None) else self.mask_algorithm_oid))
		parts.append("MaskHash=%s" % (self.mask_hash_algorithm.name if (self.mask_hash_algorithm is not None) else self.mask_hash_algorithm_oid))
		parts.append("SaltLen=%d" % (self.salt_length))
		if self.trailer_field_value is not None:
			parts.append("Trailer=#%d (0x%x)" % (self.trailer_field, self.trailer_field_value))
		else:
			parts.append("Trailer=#%d (unknown)" % (self.trailer_field))
		return "RSAPSSParameters<%s>" % (", ".join(parts))



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import hashlib
import collections
import base64
from x509sak.Tools import JSONTools
from x509sak.Exceptions import InvalidInternalDataException

class ModulusDB():
	_ModulusMatch = collections.namedtuple("ModulusMatch", [ "hash", "text" ])
	_DB_DATA = None

	def __init__(self):
		if self._DB_DATA is None:
			self._DB_DATA = self._load_db_data()

	@staticmethod
	def _hash_int(n):
		n_text = "%x" % (n)
		hashval = hashlib.md5(n_text.encode("ascii")).digest()
		return hashval

	@staticmethod
	def _decode_hash(hashstr):
		return base64.b64decode(hashstr + "==")

	def _load_db_data(self):
		db_data = { }
		for filename in [ "rsa.json", "debian.json.gz" ]:
			data = JSONTools.load_internal("x509sak.data.moduli", filename)
			for entry in data:
				if "text" not in entry:
					raise InvalidInternalDataException("Modulus database requires text for given modulus, not present for entry: %s" % (str(entry)))
				if "n" in entry:
					# Directly hash modulus
					db_data[self._hash_int(entry["n"])] = entry["text"]
				elif "md5" in entry:
					# Decode modulus
					db_data[self._decode_hash(entry["md5"])] = entry["text"]
				else:
					raise InvalidInternalDataException("Modulus database requires either 'n' or 'md5' for given modulus, not present for entry: %s" % (str(entry)))
		return db_data

	def find(self, n):
		n_hash = self._hash_int(n)
		if n_hash in self._DB_DATA:
			return self._ModulusMatch(hash = n_hash, text = self._DB_DATA[n_hash])
		else:
			return None



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

class X509SAKException(Exception): pass

# InvisibleUserException by default is not shown to the user. By default, the
# command line isn't very helpful, even though it might be the fault of the
# user.
class InvisibleUserErrorException(X509SAKException): pass
class UserErrorException(X509SAKException): pass
class ProgrammerErrorException(X509SAKException): pass

class UnfulfilledPrerequisitesException(UserErrorException): pass
class InvalidUsageException(UserErrorException): pass
class InvalidInputException(UserErrorException): pass
class UnknownFormatException(UserErrorException): pass
class InvalidCAIndexFileEntry(UserErrorException): pass
class CurveNotFoundException(UserErrorException): pass
class UnexpectedFileContentException(UserErrorException): pass
class UnsupportedEncodingException(UserErrorException): pass
class KeyCorruptException(UserErrorException): pass
class InvalidUseException(UserErrorException): pass
class InvalidIPAddressException(UserErrorException): pass

class CmdExecutionFailedException(InvisibleUserErrorException):
	def __init__(self, msg, execution_result):
		super().__init__(msg)
		self.__execution_result = execution_result

	@property
	def execution_result(self):
		return self.__execution_result

class LazyDeveloperException(ProgrammerErrorException): pass
class UnknownAlgorithmException(ProgrammerErrorException): pass
class InvalidInternalDataException(ProgrammerErrorException): pass



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2020-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import collections
from x509sak.tls.Enums import ContentType
from x509sak.tls.TLSStructs import RecordLayerPkt, AlertPkt, ServerHandshakeMessage, ClientHandshakeMessage
from x509sak.tls.Structure import DeserializationException
from x509sak.tls.DataBuffer import DataBuffer, DataBufferException

class TLSMessageDecoder():
	_TLSAlertMessage = collections.namedtuple("TLSAlertMessage", [ "encrypted", "data" ])

	def __init__(self, side):
		assert(side in [ "client", "server" ])
		self._side = side
		self._buffer = DataBuffer()
		self._msgid = 0
		self._installed_hooks = {
			"record_layer":		[ ],
			"alert":			[ ],
			"handshake":		[ ],
			"unimplemented":	[ ],
		}

	def put(self, data):
		self._buffer += data
		self._trigger_decode()

	def add_hook(self, hooktype, callback):
		self._installed_hooks[hooktype].append(callback)
		return self

	def add_hooks(self, hooktypes, callback):
		for hooktype in hooktypes:
			self.add_hook(hooktype, callback)

	def _execute_hook(self, hooktype, *data):
		for hook in self._installed_hooks[hooktype]:
			hook(hooktype, self._msgid, *data)

	def _handle_record_layer_pkt(self, record_layer_packet):
		self._execute_hook("record_layer", record_layer_packet)
		if record_layer_packet["content_type"] == ContentType.Alert:
			if len(record_layer_packet["payload"]) == 2:
				alert_packet = AlertPkt.unpack(DataBuffer(record_layer_packet["payload"]))
				self._execute_hook("alert", self._TLSAlertMessage(encrypted = False, data = alert_packet))
			else:
				# Encrypted alert
				self._execute_hook("alert", self._TLSAlertMessage(encrypted = True, data = record_layer_packet["payload"]))

		elif record_layer_packet["content_type"] == ContentType.Handshake:
			if self._side == "server":
				msg = ServerHandshakeMessage.unpack(DataBuffer(record_layer_packet["payload"]))
				self._execute_hook("handshake", msg[0], msg[1])
			else:
				msg = ClientHandshakeMessage.unpack(DataBuffer(record_layer_packet["payload"]))
				self._execute_hook("handshake", msg[0], msg[1])

		else:
			self._execute_hook("unimplemented", record_layer_packet["content_type"])

	def _trigger_decode(self):
		try:
			while True:
				record_layer_pkt = RecordLayerPkt.unpack(self._buffer)
				self._handle_record_layer_pkt(record_layer_pkt)
				self._msgid += 1
		except (DataBufferException, DeserializationException):
			# No more packets
			pass



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2018 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from x509sak import CertificatePool
from x509sak.BaseAction import BaseAction

class ActionFindCert(BaseAction):
	def __init__(self, cmdname, args):
		BaseAction.__init__(self, cmdname, args)
		self._pool = CertificatePool()
		self._pool.load_sources(self._args.crtsource)
		self._log.debug("Loaded a total of %d unique certificates in trust store.", self._pool.certificate_count)

		for cert in sorted(self._pool):
			if self._cert_matches(cert):
				cert.dump_pem()

	def _cert_matches(self, cert):
		if self._args.hashval is not None:
			if not cert.hashval.hex().startswith(self._args.hashval):
				return False
		return True



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2019 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import string

class PassphraseGenerator():
	def __init__(self, alphabets):
		self._alphabets = alphabets

	@classmethod
	def all_ascii(cls):
		return cls([ string.ascii_lowercase + string.ascii_uppercase + string.digits ])

	@classmethod
	def non_ambiguous(cls):
		remove_chars = (
			"0Oo",
			"1ilI",
			"2Z",
			"6G",
			"8B",
			"9g",
			"uv",
			"UV",
		)
		alphabet = set(string.ascii_lowercase + string.ascii_uppercase + string.digits)
		for remove_chunk in remove_chars:
			alphabet -= set(remove_chunk)
		return cls([ tuple(alphabet) ])

	@classmethod
	def pronouncible(cls):
		vovels = set("aeiou")
		consonants = set(string.ascii_lowercase) - vovels
		return cls([ tuple(consonants), tuple(vovels) ])

	@staticmethod
	def rand_int(bits):
		assert(isinstance(bits, int))
		assert(bits > 0)

		# Round up to full bytes first
		byte_cnt = (bits + 7) // 8
		rand_bytes = bytearray(os.urandom(byte_cnt))
		assert(len(rand_bytes) == byte_cnt)

		if (bits % 8) != 0:
			# Truncate most significant byte
			want_bits = bits % 8
			mask = (1 << want_bits) - 1
			rand_bytes[0] &= mask

		# Convert to integer and check we did the right thing.
		intvalue = int.from_bytes(rand_bytes, byteorder = "big")
		assert(0 <= intvalue < (2 ** bits))
		return intvalue

	# TODO: Is there a more efficient algorithm to do this? Probably so, but
	# one that involves no floating point maths?
	@staticmethod
	def max_digits_in_radix(bits, new_radix):
		assert(bits > 0)
		maxvalue = 2 ** bits
		length = 1
		curvalue = new_radix
		while curvalue < maxvalue:
			length += 1
			curvalue *= new_radix
		return length

	def max_digits(self, bits):
		assert(bits > 0)
		maxvalue = 2 ** bits
		length = 0
		curvalue = 1
		while curvalue < maxvalue:
			alphasize = len(self._alphabets[length % len(self._alphabets)])
			length += 1
			curvalue *= alphasize
		return length

	def gen_passphrase(self, bits):
		intvalue = self.rand_int(bits)
		pw_len = self.max_digits(bits)
		passphrase = [ ]
		for i in range(pw_len):
			alphabet = self._alphabets[i % len(self._alphabets)]
			(intvalue, charno) = divmod(intvalue, len(alphabet))
			passphrase.append(alphabet[charno])
		assert(intvalue == 0)
		return "".join(passphrase)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from x509sak.estimate.BaseEstimator import BaseEstimator
from x509sak.estimate import JudgementCode
from x509sak.estimate.Judgement import SecurityJudgement, SecurityJudgements

@BaseEstimator.register
class HashFunctionSecurityEstimator(BaseEstimator):
	_ALG_NAME = "hash_fnc"

	def analyze(self, hash_fnc):
		if hash_fnc.value.derating is None:
			bits_security = hash_fnc.value.output_bits / 2
		else:
			bits_security = hash_fnc.value.derating.security_lvl_bits

		judgements = SecurityJudgements()
		judgements += self.algorithm("bits").analyze(JudgementCode.X509Cert_Signature_HashFunction_DigestLengthInBits, bits_security)
		if hash_fnc.value.derating is not None:
			judgements += SecurityJudgement(JudgementCode.X509Cert_Signature_HashFunction_Derated, "Derated from ideal %d bits security level because of %s." % (hash_fnc.value.output_bits / 2, hash_fnc.value.derating.reason))

		result = {
			"name":			hash_fnc.value.name,
			"pretty":		hash_fnc.value.pretty_name,
			"bitlen":		hash_fnc.value.output_bits,
			"security":		judgements,
		}
		if self._analysis_options.include_raw_data:
			result["oid"] = str(hash_fnc.value.oid)
		return result



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import contextlib
import collections
import pyasn1.error
import pyasn1.type.univ
from pyasn1.type import tag
from pyasn1_modules import rfc2459, rfc5280
from x509sak.OID import OID, OIDDB
from x509sak import ASN1Models
from x509sak.ASN1Wrapper import ASN1GeneralNameWrapper, ASN1GeneralNamesWrapper
from x509sak.Tools import ASN1Tools
from x509sak.OtherModels import SignedCertificateTimestampList
from x509sak.tls.DataBuffer import DataBuffer
from x509sak.tls.Structure import DeserializationException
from x509sak.Exceptions import InvalidInputException
from x509sak.DistinguishedName import RelativeDistinguishedName

class X509Extensions():
	def __init__(self, extensions):
		self._exts = extensions

	def get_all(self, oid):
		assert(isinstance(oid, OID))
		return [ extension for extension in self._exts if extension.oid == oid ]

	def get_first(self, oid):
		assert(isinstance(oid, OID))
		exts = self.get_all(oid)
		if len(exts) == 0:
			return None
		else:
			return exts[0]

	def remove_all(self, oid):
		assert(isinstance(oid, OID))
		self._exts = [ extension for extension in self._exts if extension.oid != oid ]
		return self

	def filter(self, oid, replacement_extension):
		assert(isinstance(oid, OID))
		self._exts = [ extension if (extension.oid != oid) else replacement_extension for extension in self._exts ]
		return self

	def has(self, oid):
		assert(isinstance(oid, OID))
		return any(extension.oid == oid for extension in self._exts)

	def dump(self, indent = ""):
		print("%sTotal of %d X.509 extensions present:" % (indent, len(self)))
		for (eid, ext) in enumerate(self, 1):
			print("%sExtension %d: %s%s" % (indent + "    ", eid, ext.__class__.__name__, " [critical]" if ext.critical else ""))
			ext.dump(indent = indent + "        ")

	def to_asn1(self):
		extension_list = [ extension.to_asn1() for extension in self ]
		extensions_asn1 = rfc2459.TBSCertificate()["extensions"]
		extensions_asn1.setComponents(*extension_list)
		return extensions_asn1

	def __getitem__(self, index):
		return self._exts[index]

	def __iter__(self):
		return iter(self._exts)

	def __len__(self):
		return len(self._exts)

	def __str__(self):
		return "X509Extensions<%d>" % (len(self))

class X509ExtensionRegistry():
	_KNOWN_EXTENSIONS = { }
	_DEFAULT_CLASS = None

	@classmethod
	def set_default_class(cls, handler):
		cls._DEFAULT_HANDLER = handler

	@classmethod
	def install_handler_class(cls, handler):
		oid = handler.get_handler_oid()
		cls._KNOWN_EXTENSIONS[oid] = handler
		return handler

	@classmethod
	def create(cls, oid, critical, data):
		if oid in cls._KNOWN_EXTENSIONS:
			return cls._KNOWN_EXTENSIONS[oid](oid, critical, data)
		else:
			return cls._DEFAULT_HANDLER(oid, critical, data)

class X509Extension():
	_HANDLER_OID = None
	_ASN1_MODEL = None

	def __init__(self, oid, critical, data):
		assert(isinstance(oid, OID))
		assert(isinstance(critical, bool))
		assert(isinstance(data, bytes))
		self._oid = oid
		self._critical = critical
		self._data = data
		self._detailed_asn1 = None
		spec = self._ASN1_MODEL() if (self._ASN1_MODEL is not None) else None
		self._detailed_asn1 = ASN1Tools.safe_decode(self.data, asn1_spec = spec)
		self._decode_hook()

	def to_asn1(self):
		extension = rfc2459.Extension()
		extension["extnID"] = self.oid.to_asn1()
		extension["critical"] = self.critical
		extension["extnValue"] = self.data
		return extension

	@classmethod
	def construct_from_asn1(cls, asn1, critical = False):
		data = pyasn1.codec.der.encoder.encode(asn1)
		return cls(oid = cls._HANDLER_OID, data = data, critical = critical)

	@classmethod
	def get_handler_oid(cls):
		return cls._HANDLER_OID

	@property
	def asn1_model(self):
		return self._ASN1_MODEL

	@property
	def oid(self):
		return self._oid

	@property
	def critical(self):
		return self._critical

	@property
	def data(self):
		return self._data

	@property
	def detailed_asn1(self):
		return self._detailed_asn1

	@property
	def asn1(self):
		return self._detailed_asn1.asn1

	def _decode_hook(self):
		pass

	@property
	def format_value(self):
		return self.data.hex()

	@property
	def known(self):
		return self.oid in OIDDB.X509Extensions

	@property
	def name(self):
		if self.known:
			name = OIDDB.X509Extensions[self.oid]
		else:
			name = str(self.oid)
		return name

	def dump(self, indent = ""):
		print("%s%s" % (indent, str(self)))

	def __repr__(self):
		return "%s<%s = %s>" % (self.__class__.__name__, self.name, self.format_value)
X509ExtensionRegistry.set_default_class(X509Extension)


@X509ExtensionRegistry.install_handler_class
class X509SubjectKeyIdentifierExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("SubjectKeyIdentifier")
	_ASN1_MODEL = rfc2459.SubjectKeyIdentifier

	@classmethod
	def construct(cls, keyid):
		assert(isinstance(keyid, bytes))
		assert(len(keyid) == 20)
		return cls.construct_from_asn1(cls._ASN1_MODEL(keyid), critical = False)

	@property
	def keyid(self):
		return self._keyid

	@property
	def format_value(self):
		if self.keyid is not None:
			return "KeyID %s" % (self.keyid.hex())
		else:
			return "Invalid KeyID"

	def __eq__(self, other):
		return self.keyid == other.keyid

	def _decode_hook(self):
		if self.asn1 is not None:
			self._keyid = bytes(self.asn1)
		else:
			self._keyid = None


@X509ExtensionRegistry.install_handler_class
class X509AuthorityKeyIdentifierExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("AuthorityKeyIdentifier")
	_ASN1_MODEL = rfc2459.AuthorityKeyIdentifier

	@classmethod
	def construct(cls, keyid):
		assert(isinstance(keyid, bytes))
		assert(len(keyid) > 0)
		asn1 = cls._ASN1_MODEL()
		asn1["keyIdentifier"] = keyid
		return cls.construct_from_asn1(asn1, critical = False)

	@property
	def keyid(self):
		return self._keyid

	@property
	def ca_names(self):
		return self._ca_names

	@property
	def serial(self):
		return self._serial

	@property
	def format_value(self):
		values = [ ]
		if self.keyid is not None:
			values.append("KeyID %s" % (self.keyid.hex()))
		if self.ca_names is not None:
			values.append("CAName {%s}" % (", ".join(str(name) for name in self.ca_names)))
		if self.serial is not None:
			values.append("Serial %x" % (self.serial))
		return ", ".join(values)

	def _decode_hook(self):
		if self.asn1 is not None:
			if self.asn1.getComponentByName("keyIdentifier", None, instantiate = False) is not None:
				self._keyid = bytes(self.asn1["keyIdentifier"])
			else:
				self._keyid = None
			if self.asn1.getComponentByName("authorityCertIssuer", None, instantiate = False) is not None:
				self._ca_names = [ ASN1GeneralNameWrapper.from_asn1(generalname) for generalname in self.asn1["authorityCertIssuer"] ]
			else:
				self._ca_names = None
			if self.asn1.getComponentByName("authorityCertSerialNumber", None, instantiate = False) is not None:
				self._serial = int(self.asn1["authorityCertSerialNumber"])
			else:
				self._serial = None
		else:
			self._keyid = None
			self._ca_names = None
			self._serial = None


@X509ExtensionRegistry.install_handler_class
class X509BasicConstraintsExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("BasicConstraints")
	_ASN1_MODEL = rfc2459.BasicConstraints

	@property
	def pathlen(self):
		if self.asn1 is None:
			return None
		if self.asn1["pathLenConstraint"].hasValue():
			return int(self.asn1["pathLenConstraint"])
		else:
			return None

	@property
	def is_ca(self):
		if self.asn1 is not None:
			return bool(self.asn1["cA"])
		else:
			return False

	def __repr__(self):
		return "%s<CA = %s>" % (self.__class__.__name__, self.is_ca)


@X509ExtensionRegistry.install_handler_class
class X509ExtendedKeyUsageExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("ExtendedKeyUsage")
	_ASN1_MODEL = rfc2459.ExtKeyUsageSyntax

	def _decode_hook(self):
		if self.asn1 is not None:
			self._oids = [ OID.from_str(str(oid)) for oid in self.asn1 ]
		else:
			self._oids = [ ]

	@property
	def key_usage_oids(self):
		return iter(self._oids)

	@property
	def any_key_usage(self):
		return OIDDB.X509ExtendedKeyUsage.inverse("anyExtendedKeyUsage") in self._oids

	@property
	def client_auth(self):
		return OIDDB.X509ExtendedKeyUsage.inverse("id_kp_clientAuth") in self._oids

	@property
	def server_auth(self):
		return OIDDB.X509ExtendedKeyUsage.inverse("id_kp_serverAuth") in self._oids

	def __repr__(self):
		return "%s<%s>" % (self.__class__.__name__, ", ".join(OIDDB.X509ExtendedKeyUsage.get(oid, str(oid)) for oid in sorted(self._oids)))


@X509ExtensionRegistry.install_handler_class
class X509SubjectAlternativeNameExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("SubjectAlternativeName")
	_ASN1_MODEL = rfc5280.SubjectAltName

	def _decode_hook(self):
		self._known_names = [ ]
		if self.asn1 is None:
			return
		for altname in self.asn1:
			self._known_names.append(ASN1GeneralNameWrapper.from_asn1(altname))

	@property
	def name_count(self):
		return len(self._known_names)

	def get_all(self, name_type):
		return [ asn1name for asn1name in self._known_names if asn1name.name == name_type ]

	def __iter__(self):
		return iter(self._known_names)

	def __repr__(self):
		return "%s<%s>" % (self.__class__.__name__, ", ".join(str(asn1name) for asn1name in self._known_names))


@X509ExtensionRegistry.install_handler_class
class X509IssuerAlternativeNameExtension(X509SubjectAlternativeNameExtension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("IssuerAlternativeName")
	_ASN1_MODEL = rfc5280.IssuerAltName


@X509ExtensionRegistry.install_handler_class
class X509KeyUsageExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("KeyUsage")
	_ASN1_MODEL = rfc5280.KeyUsage

	@property
	def flags(self):
		"""Known flags (defined in the bitset) only."""
		return self._flags

	@property
	def all_flags(self):
		"""All set flags (also those not defined in bitset) only."""
		return self._all_flags

	@property
	def has_trailing_zero(self):
		return ASN1Tools.bitstring_has_trailing_zeros(self.asn1) if (self.asn1 is not None) else None

	@property
	def highest_set_bit_value(self):
		return ASN1Tools.bitstring_highbit(self.asn1) if (self.asn1 is not None) else None

	@property
	def highest_permissible_bit_value(self):
		return max(self._ASN1_MODEL.namedValues.values())

	@property
	def all_bits_zero(self):
		return ASN1Tools.bitstring_is_empty(self.asn1) if (self.asn1 is not None) else None

	@property
	def unknown_flags_set(self):
		return (self.highest_set_bit_value or 0) > len(self._ASN1_MODEL.namedValues)

	def _decode_hook(self):
		if self.asn1 is None:
			self._flags = None
			self._all_flags = None
		else:
			self._flags = set()
			self._all_flags = set()
			known_bits = { bit : name for (name, bit) in self._ASN1_MODEL.namedValues.items() }
			for (bit, value) in enumerate(self.asn1):
				if value == 1:
					if bit in known_bits:
						flag_name =  known_bits[bit]
						self._flags.add(flag_name)
						self._all_flags.add(flag_name)
					else:
						flag_name = "bit-%d" % (bit)
						self._all_flags.add(flag_name)

	def __repr__(self):
		if self.flags is not None:
			return "%s<%s>" % (self.__class__.__name__, ", ".join(sorted(self._flags)))
		else:
			return "%s<flags unparsable>" % (self.__class__.__name__)


@X509ExtensionRegistry.install_handler_class
class X509NetscapeCertificateTypeExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("NetscapeCertificateType")
	_ASN1_MODEL = ASN1Models.NetscapeCertificateType

	@property
	def ssl_server(self):
		return "server" in self.flags

	@property
	def ssl_client(self):
		return "client" in self.flags

	@property
	def flags(self):
		return self._flags

	def _decode_hook(self):
		self._flags = set()
		if self.asn1 is None:
			return
		for (name, bit) in self._ASN1_MODEL.namedValues.items():
			if (len(self.asn1) > bit) and self.asn1[bit]:
				self._flags.add(name)

	def __repr__(self):
		return "%s<%s>" % (self.__class__.__name__, ", ".join(sorted(self._flags)))


@X509ExtensionRegistry.install_handler_class
class X509AuthorityInformationAccessExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("id-pe-authorityInfoAccess")
	_ASN1_MODEL = rfc5280.AuthorityInfoAccessSyntax

	@property
	def method_count(self):
		return len(self._methods)

	def _decode_hook(self):
		self._methods = [ ]
		if self.asn1 is not None:
			for item in self.asn1:
				oid = OID.from_asn1(item["accessMethod"])
				location = item["accessLocation"]
				self._methods.append((oid, location))

	def __repr__(self):
		return "%s<%s>" % (self.__class__.__name__, ", ".join(str(oid) for (oid, location) in self._methods))


@X509ExtensionRegistry.install_handler_class
class X509CertificatePoliciesExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("CertificatePolicies")
	_ASN1_MODEL = rfc5280.CertificatePolicies

	_CertificatePolicy = collections.namedtuple("CertificatePolicy", [ "oid", "qualifiers" ])
	_CertificatePolicyQualifier = collections.namedtuple("CertificatePolicyQualifier", [ "oid", "qualifier_data", "decoded_qualifier" ])
	_DecodedQualifier = collections.namedtuple("DecodedQualifier", [ "asn1", "trailing_data", "constraint_violation" ])

	@property
	def policies(self):
		return iter(self._policies)

	@property
	def policy_count(self):
		return len(self._policies)

	@property
	def policy_oids(self):
		return [ policy.oid for policy in self.policies ]

	@classmethod
	def decode_qualifier(cls, oid, qualifier_data):
		decoded_qualifier = None
		if oid == OIDDB.X509ExtensionCertificatePolicyQualifierOIDs.inverse("id-qt-cps"):
			decoded_qualifier = ASN1Tools.safe_decode(qualifier_data, asn1_spec = (rfc5280.CPSuri(), ASN1Models.RelaxedCPSuri()))
		elif oid == OIDDB.X509ExtensionCertificatePolicyQualifierOIDs.inverse("id-qt-unotice"):
			decoded_qualifier = ASN1Tools.safe_decode(qualifier_data, asn1_spec = (rfc5280.UserNotice(), ASN1Models.RelaxedUserNotice()))
		return decoded_qualifier

	def get_policy(self, policy_oid):
		for policy in self._policies:
			if policy.oid == policy_oid:
				return policy
		return None

	def _decode_hook(self):
		self._policies = [ ]
		if self.asn1 is None:
			return
		for item in self.asn1:
			policy_oid = OID.from_asn1(item["policyIdentifier"])
			qualifiers = [ ]
			for qualifier in item["policyQualifiers"]:
				qualifier_oid = OID.from_asn1(qualifier["policyQualifierId"])
				qualifier_data = bytes(qualifier["qualifier"])
				qualifier = self._CertificatePolicyQualifier(oid = qualifier_oid, qualifier_data = qualifier_data, decoded_qualifier = self.decode_qualifier(qualifier_oid, qualifier_data))
				qualifiers.append(qualifier)
			policy = self._CertificatePolicy(oid = policy_oid, qualifiers = tuple(qualifiers))
			self._policies.append(policy)

	def __repr__(self):
		return "%s<%s>" % (self.__class__.__name__, ", ".join(str(oid) for (oid, qualifiers) in self._policies))


@X509ExtensionRegistry.install_handler_class
class X509CRLDistributionPointsExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("CRLDistributionPoints")
	_ASN1_MODEL = rfc5280.CRLDistributionPoints
	_KNOWN_REASON_BITS = {
		0:		"unused",
		1:		"keyCompromise",
		2:		"cACompromise",
		3:		"affiliationChanged",
		4:		"superseded",
		5:		"cessationOfOperation",
		6:		"certificateHold",
		7:		"privilegeWithdrawn",
		8:		"aACompromise",
	}
	_ALL_USED_REASONS = set(name for name in _KNOWN_REASON_BITS.values() if (name != "unused"))
	_DistributionPoint = collections.namedtuple("DistributionPoint", [ "point_name", "point_name_rdn_malformed", "reasons", "reasons_trailing_zero", "crl_issuer" ])

	@classmethod
	def all_used_reasons(cls):
		return iter(cls._ALL_USED_REASONS)

	@property
	def points(self):
		return iter(self._distribution_points)

	def _decode_hook(self):
		if self.asn1 is None:
			self._distribution_points = None
			return

		self._distribution_points = [ ]
		for asn1_point in self.asn1:
			point_name_rdn_malformed = False
			if asn1_point["distributionPoint"].hasValue():
				point_name_asn1 = asn1_point["distributionPoint"].getComponent()
				if asn1_point["distributionPoint"]["fullName"].hasValue():
					# GeneralNames
					point_name = ASN1GeneralNamesWrapper.from_asn1(point_name_asn1)
				else:
					# RelativeDistinguishedName
					try:
						point_name = RelativeDistinguishedName.from_asn1(point_name_asn1)
					except InvalidInputException:
						point_name = None
						point_name_rdn_malformed = True
			else:
				point_name = None

			if asn1_point["reasons"].hasValue():
				reasons = set()
				for (bitno, value) in enumerate(asn1_point["reasons"]):
					if value == 1:
						value = self._KNOWN_REASON_BITS.get(bitno, bitno)
						reasons.add(value)
				reasons_trailing_zero = ASN1Tools.bitstring_has_trailing_zeros(asn1_point["reasons"])
			else:
				reasons = None
				reasons_trailing_zero = False

			if asn1_point["cRLIssuer"].hasValue():
				crl_issuer = ASN1GeneralNamesWrapper.from_asn1(asn1_point["cRLIssuer"])
			else:
				crl_issuer = None

			cdp = self._DistributionPoint(point_name = point_name, point_name_rdn_malformed = point_name_rdn_malformed, reasons = reasons, reasons_trailing_zero = reasons_trailing_zero, crl_issuer = crl_issuer)
			self._distribution_points.append(cdp)

	def __repr__(self):
		if self.asn1 is not None:
			return "%s<%s>" % (self.__class__.__name__, ", ".join(str(point) for point in self.points))
		else:
			return "%s<malformed>" % (self.__class__.__name__)


@X509ExtensionRegistry.install_handler_class
class X509FreshestCRLExtension(X509CRLDistributionPointsExtension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("FreshestCRL")


@X509ExtensionRegistry.install_handler_class
class X509CertificateTransparencySCTsExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("CertificateTransparency")
	_ASN1_MODEL = pyasn1.type.univ.OctetString

	@property
	def payload(self):
		return self._payload

	@property
	def malformed_payload(self):
		return self.payload is None

	def _decode_hook(self):
		self._payload = None
		if self.asn1 is None:
			return
		raw_data = bytes(self.asn1)
		try:
			self._payload = SignedCertificateTimestampList.unpack(DataBuffer(raw_data))
		except DeserializationException:
			pass

	def __repr__(self):
		return "%s<%s>" % (self.__class__.__name__, str(self.payload))


@X509ExtensionRegistry.install_handler_class
class X509CertificateTransparencyPrecertificatePoisonExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("CertificateTransparencyPrecertificatePoison")
	_ASN1_MODEL = pyasn1.type.univ.Null

	def __repr__(self):
		return "%s<%s>" % (self.__class__.__name__, "malformed" if (self.asn1 is None) else "OK")


@X509ExtensionRegistry.install_handler_class
class X509NameConstraintsExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("NameConstraints")
	_ASN1_MODEL = rfc5280.NameConstraints
	_NameConstraintsSubtree = collections.namedtuple("NameConstraintSubtree", [ "minimum", "maximum", "base" ])

	@property
	def permitted_subtrees(self):
		return self._permitted_subtrees

	@property
	def excluded_subtrees(self):
		return self._excluded_subtrees

	def _parse_subtree(self, subtree):
		minimum = int(subtree["minimum"])
		maximum = int(subtree["maximum"]) if (subtree["maximum"].isValue) else None
		base = ASN1GeneralNameWrapper.from_asn1(subtree["base"])
		return self._NameConstraintsSubtree(minimum = minimum, maximum = maximum, base = base)

	def _parse_subtrees(self, subtrees):
		parsed_subtrees = [ ]
		for subtree in subtrees:
			parsed_subtree = self._parse_subtree(subtree)
			parsed_subtrees.append(parsed_subtree)
		return parsed_subtrees


	def _decode_hook(self):
		if self.asn1 is None:
			self._permitted_subtrees = None
			self._excluded_subtrees = None
			return
		self._permitted_subtrees = self._parse_subtrees(self.asn1["permittedSubtrees"])
		self._excluded_subtrees = self._parse_subtrees(self.asn1["excludedSubtrees"])

	def __repr__(self):
		return "%s<permitted = %s, excluded = %s>" % (self.__class__.__name__, self.permitted_subtrees, self.excluded_subtrees)


@X509ExtensionRegistry.install_handler_class
class X509SubjectDirectoryAttributesExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("SubjectDirectoryAttributes")
	_ASN1_MODEL = rfc5280.SubjectDirectoryAttributes
	_Attribute = collections.namedtuple("Attribute", [ "attribute_type", "values" ])

	@property
	def attributes(self):
		return self._attributes

	def _decode_hook(self):
		self._attributes = [ ]
		if self.asn1 is not None:
			for attribute in self.asn1:
				self._attributes.append(self._Attribute(attribute_type = OID.from_asn1(attribute["type"]), values = [ ASN1Tools.safe_decode(value) for value in attribute["values"] ]))

	def __repr__(self):
		return "%s<%s>" % (self.__class__.__name__, self.attributes)


@X509ExtensionRegistry.install_handler_class
class X509SubjectInformationAccessExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("SubjectInformationAccess")
	_ASN1_MODEL = rfc5280.SubjectInfoAccessSyntax
	_AccessDescription = collections.namedtuple("AccessDescription", [ "method", "location" ])

	@property
	def description(self):
		return self._description

	def _decode_hook(self):
		self._description = [ ]
		if self.asn1 is not None:
			for access_description in self.asn1:
				self._description.append(self._AccessDescription(method = OID.from_asn1(access_description["accessMethod"]), location = ASN1GeneralNameWrapper.from_asn1(access_description["accessLocation"])))

	def __repr__(self):
		return "%s<%s>" % (self.__class__.__name__, self.description)


@X509ExtensionRegistry.install_handler_class
class X509InhibitAnyPolicyExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("X509Version3CertificateExtensionInhibitAnyPolicy")
	_ASN1_MODEL = rfc5280.InhibitAnyPolicy

	@property
	def skipcerts(self):
		return None if (self.asn1 is None) else (int(self.asn1))

	def __repr__(self):
		return "%s<%s>" % (self.__class__.__name__, self.skipcerts)


@X509ExtensionRegistry.install_handler_class
class X509PolicyConstraintsExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("PolicyConstraints")
	_ASN1_MODEL = rfc5280.PolicyConstraints

	@property
	def require_explicit_policy(self):
		return self._require_explicit_policy

	@property
	def inhibit_policy_mapping(self):
		return self._inhibit_policy_mapping

	def _decode_hook(self):
		self._require_explicit_policy = None
		self._inhibit_policy_mapping = None
		if self.asn1 is not None:
			self._require_explicit_policy = int(self.asn1["requireExplicitPolicy"]) if self.asn1["requireExplicitPolicy"].hasValue() else None
			self._inhibit_policy_mapping = int(self.asn1["inhibitPolicyMapping"]) if self.asn1["inhibitPolicyMapping"].hasValue() else None

	def __repr__(self):
		return "%s<require explicit = %s; inhibit policy mapping = %s>" % (self.__class__.__name__, self.require_explicit_policy, self.inhibit_policy_mapping)


@X509ExtensionRegistry.install_handler_class
class X509PolicyMappingsExtension(X509Extension):
	_HANDLER_OID = OIDDB.X509Extensions.inverse("PolicyMappings")
	_ASN1_MODEL = rfc5280.PolicyMappings
	_PolicyMapping = collections.namedtuple("PolicyMapping", [ "issuer_policy", "subject_policy" ])

	@property
	def mappings(self):
		return self._mappings

	def _decode_hook(self):
		self._mappings = [ ]
		if self.asn1 is not None:
			for mapping in self.asn1:
				self._mappings.append(self._PolicyMapping(issuer_policy = OID.from_asn1(mapping["issuerDomainPolicy"]), subject_policy = OID.from_asn1(mapping["subjectDomainPolicy"])))

	def __repr__(self):
		return "%s<%s>" % (self.__class__.__name__, self.mappings)



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2021 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import re
import tempfile
from x509sak.SubprocessExecutor import SubprocessExecutor
from x509sak.Exceptions import InvalidInputException, LazyDeveloperException
from x509sak.PrivateKeyStorage import PrivateKeyStorageForm
from x509sak.WorkDir import WorkDir
from x509sak.OpenSSLConfig import OpenSSLConfig
from x509sak.TempUMask import TempUMask
from x509sak.AlgorithmDB import Cryptosystems

class OpenSSLTools():
	_EXECUTABLE = "openssl"
	_CACHED_VERSION = None
	_VERSION_RE = re.compile(r"^OpenSSL (?P<major>\d+)\.(?P<minor>\d+)\.(?P<fix>\d+)(?P<patch>[a-z])?")
	_CERT_VERIFY_REGEX = re.compile(r"error (?P<error_code>\d+) at (?P<depth>\d+) depth lookup:(?P<reason>.*)")

	@classmethod
	def openssl_version(cls):
		if cls._CACHED_VERSION is None:
			result = SubprocessExecutor([ cls._EXECUTABLE, "version" ]).run()
			match = cls._VERSION_RE.match(result.stdout_text)
			if match:
				match = match.groupdict()
				cls._CACHED_VERSION = (int(match["major"]), int(match["minor"]), int(match["fix"]), match["patch"])
		return cls._CACHED_VERSION

	@classmethod
	def __create_pem_private_key(cls, private_key_filename, keyspec):
		if keyspec.cryptosystem == Cryptosystems.RSA:
			cmd = [ cls._EXECUTABLE, "genrsa", "-out", private_key_filename, str(keyspec["bitlen"]) ]
		elif keyspec.cryptosystem == Cryptosystems.ECC_ECDSA:
			cmd = [ cls._EXECUTABLE, "ecparam", "-genkey", "-out", private_key_filename, "-name", keyspec["curvename"] ]
		elif keyspec.cryptosystem == Cryptosystems.ECC_EdDSA:
			cmd = [ cls._EXECUTABLE, "genpkey", "-out", private_key_filename, "-algorithm", keyspec["curvename"] ]
		else:
			raise LazyDeveloperException(NotImplemented, keyspec.cryptosystem)
		SubprocessExecutor(cmd).run()

	@classmethod
	def create_private_key(cls, private_key_storage, keyspec):
		with tempfile.NamedTemporaryFile(prefix = "privkey_", suffix = ".pem") as pem_file:
			cls.__create_pem_private_key(pem_file.name, keyspec)

			# Then convert to the desired result format in a second step
			cmd = [ cls._EXECUTABLE ]
			if keyspec.cryptosystem == Cryptosystems.RSA:
				cmd += [ "rsa" ]
			elif keyspec.cryptosystem == Cryptosystems.ECC_ECDSA:
				cmd += [ "ec" ]
			elif keyspec.cryptosystem in [ Cryptosystems.ECC_EdDSA, Cryptosystems.ECC_ECDH ]:
				cmd += [ "pkey" ]
			else:
				raise LazyDeveloperException(NotImplemented, keyspec.cryptosystem)
			if private_key_storage.storage_form == PrivateKeyStorageForm.PEM_FILE:
				cmd += [ "-outform", "pem" ]
			elif private_key_storage.storage_form == PrivateKeyStorageForm.DER_FILE:
				cmd += [ "-outform", "der" ]
			else:
				raise LazyDeveloperException(NotImplemented, private_key_storage.storage_form)
			cmd += [ "-in", pem_file.name, "-out", private_key_storage.full_filename ]
			SubprocessExecutor(cmd).run()

	@classmethod
	def _privkey_option(cls, private_key_storage, key_option = "key"):
		if private_key_storage.storage_form == PrivateKeyStorageForm.PEM_FILE:
			cmd = [ "-%s" % (key_option), private_key_storage.full_filename ]
		elif private_key_storage.storage_form == PrivateKeyStorageForm.DER_FILE:
			cmd = [ "-%s" % (key_option), private_key_storage.full_filename, "-keyform", "der" ]
		elif private_key_storage.storage_form == PrivateKeyStorageForm.HARDWARE_TOKEN:
			cmd = [ "-%s" % (key_option), private_key_storage.pkcs11uri, "-keyform", "engine", "-engine", "pkcs11" ]
			#cmd = [ "-%s" % (key_option), "0:%d" % (private_key_storage.key_id), "-keyform", "engine", "-engine", "pkcs11" ]
		else:
			raise LazyDeveloperException(NotImplemented, private_key_storage.storage_form)
		return cmd

	@classmethod
	def __get_config(cls, private_key_storage, x509_extensions = None, subject_alternative_dns_names = None, subject_alternative_ip_addresses = None):
		openssl_config = OpenSSLConfig()
		openssl_config.set_private_key_storage(private_key_storage)
		openssl_config.set_x509_extensions(x509_extensions)
		openssl_config.set_subject_alternative_dns_names(subject_alternative_dns_names)
		openssl_config.set_subject_alternative_ip_addresses(subject_alternative_ip_addresses)
		return openssl_config

	@classmethod
	def __create_csr_or_selfsigned_certificate(cls, private_key_storage, output_filename, subject_dn, validity_days, x509_extensions = None, subject_alternative_dns_names = None, subject_alternative_ip_addresses = None, signing_hash = None, serial = None):
		openssl_config = cls.__get_config(private_key_storage = private_key_storage, x509_extensions = x509_extensions, subject_alternative_dns_names = subject_alternative_dns_names, subject_alternative_ip_addresses = subject_alternative_ip_addresses)
		with tempfile.NamedTemporaryFile("w", prefix = "config_", suffix = ".cnf") as config_file:
			openssl_config.write_to(config_file.name)
			cmd = [ cls._EXECUTABLE, "req", "-utf8", "-new" ]
			if validity_days is not None:
				cmd += [ "-x509", "-days", str(validity_days) ]
			if signing_hash is not None:
				cmd += [ "-%s" % (signing_hash) ]
			if serial is not None:
				cmd += [ "-set_serial", "%d" % (serial) ]
			cmd += cls._privkey_option(private_key_storage)
			cmd += [  "-subj", subject_dn, "-out", output_filename ]
			SubprocessExecutor(cmd, env = { "OPENSSL_CONF": config_file.name }).run()

	@classmethod
	def create_selfsigned_certificate(cls, private_key_storage, certificate_filename, subject_dn, validity_days, x509_extensions = None, subject_alternative_dns_names = None, subject_alternative_ip_addresses = None, signing_hash = None, serial = None):
		return cls.__create_csr_or_selfsigned_certificate(
				private_key_storage = private_key_storage,
				output_filename = certificate_filename,
				subject_dn = subject_dn,
				validity_days = validity_days,
				x509_extensions = x509_extensions,
				subject_alternative_dns_names = subject_alternative_dns_names,
				subject_alternative_ip_addresses = subject_alternative_ip_addresses,
				signing_hash = signing_hash,
				serial = serial,
		)

	@classmethod
	def create_csr(cls, private_key_storage, csr_filename, subject_dn, x509_extensions = None, subject_alternative_dns_names = None, subject_alternative_ip_addresses = None, signing_hash = None):
		return cls.__create_csr_or_selfsigned_certificate(
				private_key_storage = private_key_storage,
				output_filename = csr_filename,
				subject_dn = subject_dn,
				validity_days = None,
				x509_extensions = x509_extensions,
				subject_alternative_dns_names = subject_alternative_dns_names,
				subject_alternative_ip_addresses = subject_alternative_ip_addresses,
				signing_hash = signing_hash,
		)

	@classmethod
	def ca_sign_csr(cls, ca_manager, csr_filename, crt_filename, subject_dn, validity_days, x509_extensions = None, subject_alternative_dns_names = None, subject_alternative_ip_addresses = None, signing_hash = None):
		csr_absfilename = os.path.realpath(csr_filename)
		crt_absfilename = os.path.realpath(crt_filename)
		openssl_config = cls.__get_config(private_key_storage = ca_manager.private_key_storage, x509_extensions = x509_extensions, subject_alternative_dns_names = subject_alternative_dns_names, subject_alternative_ip_addresses = subject_alternative_ip_addresses)
		with WorkDir(ca_manager.capath), tempfile.NamedTemporaryFile("w", prefix = "config_", suffix = ".cnf") as config_file:
			openssl_config.write_to(config_file.name)
			cmd = [ cls._EXECUTABLE, "ca", "-utf8", "-in", csr_absfilename, "-batch", "-notext", "-out", crt_absfilename ]
			cmd += cls._privkey_option(ca_manager.private_key_storage, key_option = "keyfile")
			if subject_dn is not None:
				cmd += [ "-subj", subject_dn ]
			if validity_days is not None:
				cmd += [ "-days", str(validity_days) ]
			if signing_hash is not None:
				cmd += [ "-md", signing_hash ]
			SubprocessExecutor(cmd, env = { "OPENSSL_CONF": config_file.name }).run()

	@classmethod
	def ca_revoke_crt(cls, ca_manager, crt_filename):
		crt_absfilename = os.path.realpath(crt_filename)
		openssl_config = cls.__get_config(private_key_storage = ca_manager.private_key_storage)
		with WorkDir(ca_manager.capath), tempfile.NamedTemporaryFile("w", prefix = "config_", suffix = ".cnf") as config_file:
			openssl_config.write_to(config_file.name)
			cmd = [ cls._EXECUTABLE, "ca", "-revoke", crt_absfilename ]
			cmd += cls._privkey_option(ca_manager.private_key_storage, key_option = "keyfile")
			SubprocessExecutor(cmd, env = { "OPENSSL_CONF": config_file.name }).run()

	@classmethod
	def ca_create_crl(cls, ca_manager, crl_filename, signing_hash = None, validity_days = None):
		crl_absfilename = os.path.realpath(crl_filename)
		openssl_config = cls.__get_config(private_key_storage = ca_manager.private_key_storage)
		with WorkDir(ca_manager.capath), tempfile.NamedTemporaryFile("w", prefix = "config_", suffix = ".cnf") as config_file:
			openssl_config.write_to(config_file.name)
			cmd = [ cls._EXECUTABLE, "ca", "-gencrl", "-out", crl_absfilename ]
			if validity_days is not None:
				cmd += [ "-crldays", str(validity_days) ]
			if signing_hash is not None:
				cmd += [ "-md", signing_hash ]
			cmd += cls._privkey_option(ca_manager.private_key_storage, key_option = "keyfile")
			SubprocessExecutor(cmd, env = { "OPENSSL_CONF": config_file.name }).run()

	@classmethod
	def sign_data(cls, signing_algorithm, private_key_filename, payload):
		cmd = [ cls._EXECUTABLE, "dgst", "-sign", private_key_filename, "-%s" % (signing_algorithm.value.hash_fnc.value.name) ]
		signature = SubprocessExecutor(cmd, stdin = payload).run().stdout
		return signature

	@classmethod
	def private_to_public(cls, private_key_filename, public_key_filename):
		success = SubprocessExecutor([ cls._EXECUTABLE, "rsa", "-in", private_key_filename, "-pubout", "-out", public_key_filename ], on_failure = "pass").run().successful
		if not success:
			success = SubprocessExecutor([ cls._EXECUTABLE, "ec", "-in", private_key_filename, "-pubout", "-out", public_key_filename ], on_failure = "pass").run().successful
		if not success:
			raise InvalidInputException("File %s contained neither RSA nor ECC private key." % (private_key_filename))

	@classmethod
	def create_pkcs12(cls, certificates, private_key_storage = None, modern_crypto = True, passphrase = None):
		with TempUMask(), tempfile.NamedTemporaryFile("w", prefix = "pkcs_pass_", suffix = ".txt") as pass_file:
			if passphrase is not None:
				print(passphrase, file = pass_file)
			else:
				print(file = pass_file)
			pass_file.flush()

			cmd = [ cls._EXECUTABLE, "pkcs12", "-export" ]
			if private_key_storage is None:
				cmd += [ "-nokeys" ]
			else:
				assert(private_key_storage.is_file_based)
				if private_key_storage.storage_form == PrivateKeyStorageForm.PEM_FILE:
					cmd += [ "-inkey", private_key_storage.filename ]
				else:
					raise LazyDeveloperException(NotImplemented, private_key_storage.storage_form)
			cmd += [ "-passout", "file:%s" % (pass_file.name) ]
			if modern_crypto:
				cmd += [ "-macalg", "sha384", "-maciter", "-keypbe", "aes-128-cbc" ]
			pem_certificates = "\n".join(certificate.to_pem_data() for certificate in certificates)
			output = SubprocessExecutor(cmd, stdin = pem_certificates.encode("ascii")).run().stdout
			return output

	@classmethod
	def get_tls_server_cert_pem(cls, hostname, port = 443):
		result = SubprocessExecutor([ cls._EXECUTABLE, "s_client", "-connect", "%s:%d" % (hostname, port), "-servername", hostname ]).run()
		return result.stdout.decode()

	@classmethod
	def validate_signature(cls, issuer_certificate, subject_certificate, verbose_failure = False):
		with tempfile.NamedTemporaryFile(prefix = "subject_", suffix = ".crt") as subject, tempfile.NamedTemporaryFile(prefix = "issuer_", suffix = ".crt") as issuer, tempfile.TemporaryDirectory(prefix = "empty") as emptydir:
			subject_certificate.write_pemfile(subject.name)
			issuer_certificate.write_pemfile(issuer.name)

			cmd = [ "openssl", "verify", "-CApath", emptydir ]
			if cls.openssl_version() >= (1, 1, 0, ""):
				cmd += [ "-no_check_time" ]
			cmd += [ "-check_ss_sig", "-CAfile", issuer.name, subject.name ]
			result = SubprocessExecutor(cmd, on_failure = "pass").run()
			if result.successful:
				return True
			else:
				# Maybe the certificate signature was okay, but the complete
				# chain couldn't be established. This would still count as a
				# successful verification, however.
				match = cls._CERT_VERIFY_REGEX.search(result.stdouterr_text)
				if match:
					match = match.groupdict()
					(error_code, depth) = (int(match["error_code"]), int(match["depth"]))
					chain_valid = (error_code == 2) and (depth == 1)
					if (not chain_valid) and verbose_failure:
						print("Certificate verification error, error_code %d, depth = %d. %s not signed by %s." % (error_code, depth, subject_certificate, issuer_certificate))
						result.dump()
					return chain_valid
				else:
					# If in doubt, reject.
					if verbose_failure:
						print("Certificate verification error. %s not signed by %s." % (subject_certificate, issuer_certificate))
						result.dump()
					return False



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import urllib.parse
import pyasn1.codec.der.encoder
from pyasn1_modules import rfc5280
from x509sak.DistinguishedName import DistinguishedName
from x509sak.IPAddress import IPAddressSubnet
from x509sak.OID import OID

class ASN1GeneralNameWrapper():
	KNOWN_TYPE_NAMES = set([ "otherName", "rfc822Name", "dNSName", "x400Address", "directoryName", "ediPartyName", "uniformResourceIdentifier", "iPAddress", "registeredID" ])

	def __init__(self, name, asn1_value):
		assert(isinstance(name, str))
		assert(name in self.KNOWN_TYPE_NAMES)
		self._name = name
		self._asn1_value = asn1_value
		self._cached = None

	@property
	def name(self):
		return self._name

	@property
	def asn1_value(self):
		return self._asn1_value

	@property
	def directory_name(self):
		assert(self.name == "directoryName")
		if self._cached is None:
			self._cached = DistinguishedName.from_asn1(self.asn1_value)
		return self._cached

	@property
	def uri(self):
		assert(self.name == "uniformResourceIdentifier")
		if self._cached is None:
			self._cached = urllib.parse.urlparse(self.str_value)
		return self._cached

	@property
	def ip(self):
		assert(self.name == "iPAddress")
		if self._cached is None:
			self._cached = IPAddressSubnet.from_bytes(self.asn1_value, allow_ip_only = True)
		return self._cached

	@property
	def registered_id(self):
		assert(self.name == "registeredID")
		if self._cached is None:
			self._cached = OID.from_asn1(self.asn1_value)
		return self._cached

	@property
	def str_value(self):
		if self.name in [ "dNSName", "rfc822Name", "uniformResourceIdentifier" ]:
			result = str(self.asn1_value)
		elif (self.name == "iPAddress") and (len(self.asn1_value) in [ 4, 8, 16, 32 ]):
			result = str(self.ip)
		elif self.name == "otherName":
			oid = self.asn1_value["type-id"]
			inner_value = bytes(self.asn1_value["value"])
			result = "otherName:%s:#%s" % (oid, inner_value.hex())
		elif self.name == "directoryName":
			result = self.directory_name.pretty_str
		elif self.name == "registeredID":
			return str(self.registered_id)
		else:
			result = "%s:#%s" % (self._name, pyasn1.codec.der.encoder.encode(self.asn1_value).hex())
		return result

	@classmethod
	def from_asn1(cls, general_name):
		for name in rfc5280.GeneralName.componentType:
			value = general_name.getComponentByName(name, None, instantiate = False)
			if value is not None:
				return cls(name, value)
		assert(False)

	def __eq__(self, other):
		return (self.name, self.asn1_value) == (other.name, other.asn1_value)

	def __neq__(self, other):
		return not (self == other)

	def __repr__(self):
		return "%s = %s" % (self.name, self.str_value)

class ASN1GeneralNamesWrapper():
	def __init__(self, general_names):
		self._names = general_names

	def filter_by_type(self, typename):
		assert(typename in ASN1GeneralNameWrapper.KNOWN_TYPE_NAMES)
		return ASN1GeneralNamesWrapper([ general_name for general_name in self if (general_name.name == typename) ])

	def get_contained_uri_scheme_set(self, ignore_case = True):
		uri_schemes = set()
		for general_name in self.filter_by_type("uniformResourceIdentifier"):
			uri_scheme = general_name.uri.scheme
			if ignore_case:
				uri_scheme = uri_scheme.lower()
			uri_schemes.add(uri_scheme)
		return uri_schemes

	@classmethod
	def from_asn1(cls, general_names):
		wrapped_general_names = [ ]
		for general_name_asn1 in general_names:
			wrapped_general_names.append(ASN1GeneralNameWrapper.from_asn1(general_name_asn1))
		return cls(wrapped_general_names)

	def __iter__(self):
		return iter(self._names)

	def __len__(self):
		return len(self._names)

	def __repr__(self):
		return "GeneralNames<%s>" % (", ".join(str(name) for name in self))



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2018 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import os
import tempfile
from x509sak.OpenSSLTools import OpenSSLTools
from x509sak.CAManager import CAManager
from x509sak.BaseAction import BaseAction
from x509sak.PrivateKeyStorage import PrivateKeyStorage
from x509sak.Exceptions import CmdExecutionFailedException, UnfulfilledPrerequisitesException, InvalidInputException
from x509sak.KeySpecification import KeySpecification

class ActionCreateCSR(BaseAction):
	def __init__(self, cmdname, args):
		BaseAction.__init__(self, cmdname, args)
		if os.path.exists(self._args.out_filename) and (not self._args.force):
			raise UnfulfilledPrerequisitesException("File/directory %s already exists. Remove it first or use --force." % (self._args.out_filename))
		if (self._args.gen_keyspec is not None) and (self._args.keytype == "hw"):
			raise InvalidInputException("x509sak cannot generate private keys on a hardware token; please do this with a different tool and use x509sak to then use the created key.")
		if (cmdname.lower() in [ "gencrt", "generatecrt" ]) and (self._args.create_crt is None):
			raise InvalidInputException("When creating a certificate instead of a CSR, you need to specify the signing CA.")

		private_key_storage = PrivateKeyStorage.from_str(self._args.keytype, self._args.key_filename)
		gen_keyspec = self._args.gen_keyspec or KeySpecification.from_cmdline_str("ecc:secp384r1")
		if (private_key_storage.is_file_based) and (not os.path.exists(private_key_storage.filename)):
			OpenSSLTools.create_private_key(private_key_storage, gen_keyspec)

		custom_x509_extensions = { custom_x509_extension.key: custom_x509_extension.value for custom_x509_extension in self._args.extension }
		if self._args.create_crt is None:
			# Create CSR
			CAManager.create_csr(private_key_storage, self._args.out_filename, self._args.subject_dn, custom_x509_extensions = custom_x509_extensions, extension_template = self._args.template, subject_alternative_dns_names = self._args.san_dns, subject_alternative_ip_addresses = self._args.san_ip, signing_hash = self._args.hashfnc)
		else:
			# Create certificate
			with tempfile.NamedTemporaryFile(prefix = "csr_", suffix = ".pem") as csr:
				OpenSSLTools.create_csr(private_key_storage, csr.name, subject_dn = "/CN=Discard")
				ca = CAManager(self._args.create_crt)
				try:
					ca.sign_csr(csr.name, self._args.out_filename, subject_dn = self._args.subject_dn, custom_x509_extensions = custom_x509_extensions, validity_days = self._args.validity_days, extension_template = self._args.template, subject_alternative_dns_names = self._args.san_dns, subject_alternative_ip_addresses = self._args.san_ip, signing_hash = self._args.hashfnc)
				except CmdExecutionFailedException:
					self._log.error("Error creating certificate; is the common name you requested maybe already in use in the CA's database?")
					raise



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2020-2020 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

import pprint
from x509sak.BaseAction import BaseAction
from x509sak.tls.TLSStructs import CertificatePkt
from x509sak.tls.TLSMessageDecoder import TLSMessageDecoder
from x509sak.X509Certificate import X509Certificate
from x509sak.HexDump import HexDump

class ActionTLSParse(BaseAction):
	def __init__(self, cmdname, args):
		BaseAction.__init__(self, cmdname, args)

		if args.encoding == "hex":
			with open(args.filename) as f:
				data = bytes.fromhex(f.read().replace("\n", ""))
		elif args.encoding == "bin":
			with open(args.filename, "rb") as f:
				data = f.read()
		else:
			raise NotImplementedError(args.encoding)

		if self._args.verbose >= 1:
			hd = HexDump()
			hd.dump(data)

		self._pp = pprint.PrettyPrinter(width = 180)
		decoder = TLSMessageDecoder(side = args.side)
		decoder.add_hook("alert", self._msg_alert)
		decoder.add_hook("handshake", self._msg_handshake)
		decoder.put(data)

	def _msg_alert(self, hooktype, msg):
		print("Alert:", msg)

	def _msg_handshake(self, hooktype, msg_id, msg_type, data):
		print("%s:" % (msg_type.name))
		self._pp.pprint(data)
		if msg_type == CertificatePkt:
			print("%d certificates received:" % (len(data["payload"]["certificates"])))
			for der_data in data["payload"]["certificates"]:
				cert = X509Certificate(bytes(der_data))
				print(cert)
		print()



#	x509sak - The X.509 Swiss Army Knife white-hat certificate toolkit
#	Copyright (C) 2018-2021 Johannes Bauer
#
#	This file is part of x509sak.
#
#	x509sak is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; this program is ONLY licensed under
#	version 3 of the License, later versions are explicitly excluded.
#
#	x509sak is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with x509sak; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	Johannes Bauer <JohannesBauer@gmx.de>

from x509sak import CertificatePool
from x509sak.tests import BaseTest

class CertificatePoolTests(BaseTest):
	def test_empty_pool(self):
		pool = CertificatePool()
		server_cert = self._load_crt("ok/johannes-bauer.com")
		intermediate_cert = self._load_crt("ok/johannes-bauer-intermediate")
		root_cert = self._load_crt("ok/johannes-bauer-root")

		chain = pool.find_chain(server_cert)
		self.assertEqual(chain.root, None)
		self.assertEqual(chain.chain, tuple())
		self.assertEqual(chain.leaf, server_cert)
		self.assertEqual(chain.full_chain, (server_cert, ))

		chain = pool.find_chain(intermediate_cert)
		self.assertEqual(chain.root, None)
		self.assertEqual(chain.chain, tuple())
		self.assertEqual(chain.leaf, intermediate_cert)
		self.assertEqual(chain.full_chain, (intermediate_cert, ))

		chain = pool.find_chain(root_cert)
		self.assertEqual(chain.root, root_cert)
		self.assertEqual(chain.chain, tuple())
		self.assertEqual(chain.leaf, root_cert)
		self.assertEqual(chain.full_chain, (root_cert, ))

	def test_fullchain(self):
		pool = CertificatePool()
		server_cert = self._load_crt("ok/johannes-bauer.com")
		intermediate_cert = self._load_crt("ok/johannes-bauer-intermediate")
		root_cert = self._load_crt("ok/johannes-bauer-root")
		pool.add_certificate(server_cert)
		pool.add_certificate(intermediate_cert)
		pool.add_certificate(root_cert)

		self.assertEqual(list(pool.find_issuers(server_cert))[0], intermediate_cert)
		self.assertEqual(list(pool.find_issuers(intermediate_cert))[0], root_cert)
		self.assertEqual(list(pool.find_issuers(root_cert))[0], root_cert)

		chain = pool.find_chain(server_cert)
		self.assertEqual(chain.root, root_cert)
		self.assertEqual(chain.chain, (intermediate_cert, ))
		self.assertEqual(chain.leaf, server_cert)
		self.assertEqual(chain.full_chain, (server_cert, intermediate_cert, root_cert))

	def test_partial_chain(self):
		pool = CertificatePool()
		server_cert = self._load_crt("ok/johannes-bauer.com")
		intermediate_cert = self._load_crt("ok/johannes-bauer-intermediate")
		pool.add_certificate(server_cert)
		pool.add_certificate(intermediate_cert)

		chain = pool.find_chain(server_cert)
		self.assertEqual(chain.root, None)
		self.assertEqual(chain.chain, (intermediate_cert, ))
		self.assertEqual(chain.leaf, server_cert)
		self.assertEqual(chain.full_chain, (server_cert, intermediate_cert))
